#include <formatio.h>
#include <visa.h>
#include <stdlib.h>  
#include <stdio.h>  
#include <math.h>
#include "AVR3271.h"

#define AVR3271_REVISION  "Rev 1.0, 04/98, CVI 4.0.1" /*Instrument driver revision*/
#define BUFFER_SIZE    512L              /* File I/O buffer size */

/*==================== ADVANTEST R3271 Spectrum Analyzer ====================*/
/*                                                                           */
/* LabWindows/CVI 4.0.1 Instrument Driver                                    */
/* Original Release: APRIL 1998                                              */
/* By: Denis BIRINGER, Advantest Europe R&D S.A.R.L. Les ULIS France         */
/*     PH. + 33 1 69 18 25 94        Fax + 33 1 69 18 25 90                  */
/*     e-mail: d.biringer@erd.advantest.de                                   */
/*                                                                           */
/* Originally written in C                                                   */
/* Modification History: None                                                */
/*===========================================================================*/
 
/*****************************************************************************/
/* INSTRUMENT-DEPENDENT COMMAND ARRAYS                                       */
/*****************************************************************************/
static ViString s_init[] = {"DL0","HD0","TYPE?"};

static ViString s_freqType[] = {"CF","FA","FB"};         
static ViString s_spanType[] = {"SP","FS","ZS"};         
static ViString s_freqOffset[] = {"FOF","FON"};
static ViString s_freqStepSize[] = {"CS","CA"};
static ViString s_counterResolution[] = {"CN0","CN1","CN2","CN3"};
static ViString s_counterONOFF[] = {"COUNT OFF","COUNT ON"};

static ViString s_refLevel[] = {"RL"};
static ViString s_refUnit[] = {"UB","UM","UU","UE","UW","","KSD","AUNITS W"};
static ViString s_yScale[] = {"DD10DB","DD5DB","DD2DB","DD1DB","DD0.5DB","DD0.2DB","DD0.1DB",
							  "LIN1","LIN2","LIN5","LIN10"};
static ViString s_refLevelOffset[] = {"ROF","RON"};
static ViString s_att[] = {"AT","AA"};
static ViString s_attMin[] = {"ATMIN ON","ATMIN OFF"}; 
static ViString s_highsence[] = {"HS OFF","HS ON"};

static ViString s_RBW[] = {"BA","RB10HZ","RB30HZ","RB100HZ","RB300HZ","RB1KZ","RB3KZ","RB10KZ",
                           "RB30KZ","RB100KZ","RB300KZ","RB1MZ","RB3MZ"};                    
static ViString s_VBW[] = {"VA","VB1HZ","VB3HZ","VB10HZ","VB30HZ","VB100HZ","VB300HZ","VB1KZ",
                           "VB3KZ","VB10KZ","VB30KZ","VB100KZ","VB300KZ","VB1MZ","VB3MZ"};                

static ViString s_All_Coupled[] = {"AL"};

static ViString s_sweepTime[] = {"SW","AS"};
static ViString s_sweepMode[] = {"SI","SN","SR","TS"}; 
static ViString s_windowSweep[] = {"WDOSWP OFF","WDOSWP ON"};
static ViString s_triggerTV[] = {"TVV","TVHNT","TVHPS","TVPOL-","TVPOL+","TVH"};
static ViString s_trigger[] = {"FR","LI","VI","EXT","TRIGSLP-","TRIGSLP+","TRPOSI"};

static ViString s_traceAMode[] = {"AW","AV","AB","AM","AMIN","AG"};
static ViString s_averageMode[] = {"AG1","AG0"};  
static ViString s_traceBMode[] = {"BW","BV","BB","BM","BMIN","BG"};                            
static ViString s_detectorMode[] = {"DTN","DTP","DTG","DTS"};

static ViString s_showDL[] = {"DLF","DLN"};

static ViString s_LLType[] = {"LIMTYP TIME","LIMTYP FREQ"};
static ViString s_deleteLL[] = {"LMTBDEL","LMTADEL"};
static ViString s_setLL[] = {"LMTBIN","LMTAIN"};
static ViString s_showLL1[] = {"LAF","LAN"};
static ViString s_showLL2[] = {"LBF","LBN"}; 
static ViString s_levelX[] = {"LIMPOS ABS","LIMPOS REL"};
static ViString s_levelY[] = {"LIMAPOS ABS","LIMAPOS REL"};
static ViString s_shiftLL[] = {"LIMASFT","LIMSFT"};
static ViString s_LLpassfail[] = {"PFC OFF","PFC ON"};
static ViString s_LLjudgement[] = {"OPF?"};
static ViString s_judgementText[] = {"PASS","UPPER FAIL","LOWER FAIL","UPPER&LOWER FAIL"};

static ViString s_positionWindow[]  = {"WLX","WDX","WUL","WLL"};
static ViString s_showWindow[]  = {"WDO OFF","WDO ON"};

static ViString s_showLC[]  = {"CR OFF","CR ON"};
static ViString s_workLC[]  = {"CRDEL","CRIN"};

static ViString s_mkrNormal[] = {"MKN"};
static ViString s_mkrDelta[] = {"MKD"};
static ViString s_fixedMkrDelta[] = {"FXF","FXN"};
static ViString s_1_deltaMkr[] = {"REDLT OFF","REDLT ON"};
static ViString s_mkrTrace[] = {"MKTRACE TRB","MKTRACE TRA"};
static ViString s_mkrNDstatus[] = {"markers inactive","only normal marker active",
                                   "normal and delta markers active"};
static ViString s_multiMkrStatus[] = {"MLT"}; 
static ViString s_multiMkrON[] = {"MLN1","MLN2","MLN3","MLN4","MLN5","MLN6"};
static ViString s_multiMkrOFF[] = {"MLF1","MLF2","MLF3","MLF4","MLF5","MLF6"};
static ViString s_deleteMarker[] = {"MO"};
                                  
static ViString s_continuousSearch[] = {"CPF","CPN"};
static ViString s_signalTrack[] = {"SGF","SGN"};
static ViString s_peakRange[] = {"PSN","PSU","PSL","DY"};
static ViString s_peakSearch[] = {"MKPK","NXP","NXL","NXR","MIS","NXM"};
static ViString s_markerDeltaTo[] = {"MTCF","MTSP","MTCS","MTMKS"};
static ViString s_peakTo[] = {"MKRL","MKCF"};

static ViString s_traceMath[] = {"","ACHB","TR2","TR0","TR1"};
static ViString s_NormMode[] = {"AN ON","AR","AI","AN OFF"};
static ViString s_traceAverage[]  = {"AGR","AGS","AGP","AGC"};


static ViString s_readFreq[]= {"CF?","SP?","FA?","FB?","FO?"};
static ViString s_readFreqStepSize[] = {"CS?","CA?"};
static ViString s_readFreqCounter[] = {"COUNT?","MF?"};

static ViString s_readRefLevel[]={"RL?"};
static ViString s_readRefUnit[]={"AUNITS?"};
static ViString s_readUnitName[] = {"dBm","dBmV","dBV","dBVemf","dBpW","","V","W"};
static ViString s_readRefLevelOffset[]={"RO?"};

static ViString s_readSweepTime[] = {"SW?","AS?"};
static ViString s_readSweepMode[] = {"SWM?","Continuous&full","Single&full",
                                     "Continuous&window","Single&window"};
static ViString s_readTrigger[] = {"TM?","Free Run","TV_V","Video","TV_H","External",
                                   "VI?","TVH?","EXT?","TRPOSI?"};

static ViString s_readRBWandVBW[] = {"VB?","RB?","VA?","BA?"};
static ViString s_readAtt[] = {"AT?","AA?"};
static ViString s_readScale[] = {"10dB/","5dB/","2dB/","1dB/","0.5dB/","0.2dB/","0.1dB/",
								 "LINx1","LINx2","LINx5","LINx10"};
                                  
static ViString s_readMarkerTraceStatus[] = {"MKTRACE?","blank","Trace A","Trace B"};                                  
static ViString s_mkrNDsearch[] = {"MN?","MF?","ML?"};                                    
static ViString s_readMKDoptions[] = {"FX?","REDLT?"};                                   
static ViString s_multiMkrSearch[] = {"MLSF?","MLSL?"};
static ViString s_readmultiMkrStatus[] = {"MLT?"};
static ViString s_multiMkrMessage[] = {"multimarkers inactive","multimarkers active"};                                  
                                  
static ViString s_readPeakOptions[] = {"CP?","SG?","DY?"};
static ViString s_readDisplayLine[] = {"DL?"};
static ViString s_readLLoptions[] = {"LIMAPOS?","LIMPOS?"};
static ViString s_readLLmessage[] = {"Absolute","Relative"};
static ViString s_readMeasWindow[] = {"WLX?","WDX?","WLY?","WDY?","WDO?"};
static ViString s_readDetectorMode[] = {"DM?","Normal","Positive","Negative","Sample"};
static ViString s_readLLstatus[] = {"LMTB?","LMTA?"};
static ViString s_readTraceStatus[] = {"TB?","TA?"};
static ViString s_readTraceMessage[] = {"B_Write","B_View","B_Blank","A_Write","A_View",
                                        "A_Blank","A_norm","A-DL->A","A-B->A","B-A->A",
                                        "A_Max","A_Avg"};
static ViString s_readLevelCorrection[] = {"CR?"};                                        

static ViString s_textONOFF[] = {"inactive","active"};
static ViString s_readDisplayMode[] = {"absolute","relative"};
                                  
static ViString s_traceBinary[] = {"TBB?","TBA?"};
static ViString s_traceASCII[] = {"TAB?","TAA?"};
static ViString s_traceAccuracy[] = {"TPF","TPC","TP?"};
                                  
static ViString s_calLevel[] = {"CLN","CLF"};                                  
static ViString s_calibration[]  = {"CLA","CLG","IT0","IT1","IT2","IT3","IT4","IT6"};
static ViString s_freqCorrection[] = {"FC OFF","FC ON"};
static ViString s_calCorrection[]  = {"CC OFF","CC ON"};

static ViString s_clearStatusByte[] = {"S2"};
static ViString s_label[] = {"LBDSP OFF","LOF","LBDSP ON","LON/"};

static ViString s_initCard[] = {"MMI /:B/","MMI /A:/"};
static ViString s_copyCard[] = {"ALLCOPY /B: A:/","ALLCOPY /A: B:/"};
static ViString s_selectCard[] = {"CDRB","CDRA"};
static ViString s_recallFile[] = {"RC /B:","RC /A:"};
static ViString s_saveFile[] = {"SV /B:","SV /A:"};

static ViString s_reset[]={"IP","MND OFF"};
static ViString s_errorQuery[]={"ERRNO?"};
static ViString s_instrRevision[] = {"REV?"};

static ViString s_freqUnit[] = {"Hz","kHz","MHz","GHz"};
static ViString s_timeUnit[] = {"ns","us","ms","s"};
static ViString s_levelUnit[] = {"DB","DB","DB","DB","DB","","MV","MW"};
static ViString s_positionUnit[] = {"SC","HZ"};

static ViString s_close[] = {"LOCAL"};

static ViString s_displayMode[] = {"MDA","MDR"};

/*****************************************************************************/
/* INSTRUMENT-DEPENDENT STATUS/RANGE STRUCTURE                               */
/*****************************************************************************/

struct AVR3271_stringValPair {
    ViStatus stringVal;
    ViString stringName;
};

struct AVR3271_instrFeatures {
    ViReal64 frequencyLow[3];
    ViReal64 frequencyHigh[3];
    ViReal64 spanLow;
    ViReal64 spanHigh;
    ViReal64 freqOffsetMin;
    ViReal64 freqOffsetMax;
    ViReal64 inputAttMin;
    ViReal64 inputAttMax;
    ViReal64 refLevelMin[8];
    ViReal64 refLevelMax[8];
    ViReal64 refLevelOffsetMin;
    ViReal64 refLevelOffsetMax;
    ViReal64 sweepTimeMini[2];
    ViReal64 sweepTimeMaxi[2];
    ViReal64 gatePositionMini;
    ViReal64 gatePositionMaxi;
    ViReal64 gateWidthMini;
    ViReal64 gateWidthMaxi;
    ViInt16  averageCountMini;
    ViInt16  averageCountMaxi;
    ViInt16  DL_LevelMin;
    ViInt16  DL_LevelMax;
    ViInt16  displayMode;
    ViInt16  minAttControl;
    ViInt16  refUnit;
    ViInt16  span;
    ViInt16  scale;
    ViUInt32 instrumentType;
    ViReal64 deltaYmin;
    ViReal64 deltaYmax;
    ViReal64 calLevelMin;
    ViReal64 calLevelMax;
    ViInt16  MKRMstatus[8]; /*0 if multi-marker inactive, 1 if muti-marker active*/
    ViChar   instrDriverRevision[256];
    
};

typedef struct AVR3271_stringValPair AVR3271_tStringValPair;
    
typedef struct AVR3271_instrFeatures *AVR3271_ptrFeatures;

/*****************************************************************************/
/* UTILITY ROUTINE DECLARATIONS (Non-Exportable Functions)                   */
/*****************************************************************************/

ViBoolean AVR3271_invalidViBooleanRange (ViBoolean val);
ViBoolean AVR3271_invalidViInt16Range (ViInt16 val, ViInt16 min, ViInt16 max);
ViBoolean AVR3271_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max);
ViBoolean AVR3271_invalidViUInt16Range (ViUInt16 val, ViUInt16 min, ViUInt16 max);
ViBoolean AVR3271_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max);
ViBoolean AVR3271_invalidViReal32Range (ViReal32 val, ViReal32 min, ViReal32 max);
ViBoolean AVR3271_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max);
ViStatus AVR3271_initCleanUp (ViSession openRMSession,ViPSession openInstrSession,
                             ViStatus currentStatus);
ViStatus AVR3271_defaultInstrSetup (ViSession instrSession, ViUInt32 instrType);
ViStatus AVR3271_initReset (ViSession, ViUInt32);
ViStatus AVR3271_Search_Scale (ViSession instrSession, ViPInt16 scale);
                  
/*****************************************************************************/
/* USER-CALLABLE FUNCTIONS (Exportable Functions)                            */
/*****************************************************************************/


/*===========================================================================*/
/* Function: Initialize                                                      */
/* Purpose:  This function opens the instrument, queries the instrument      */
/*           for its ID, and initializes the instrument to a known state.    */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_init (ViRsrc resourceName, ViBoolean IDquery,
                                ViBoolean reset, ViPSession instrSession)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViSession rmSession = 0;
    ViUInt32 retCnt = 0;
    ViByte Buffer[BUFFER_SIZE];
    ViByte interim[5];
    ViUInt32 AVR3271_instrType,retCount;
    ViByte cable;
    
    /*- Check input parameter ranges ----------------------------------------*/

    if (AVR3271_invalidViBooleanRange (IDquery))
        return VI_ERROR_PARAMETER2;
        
    if (AVR3271_invalidViBooleanRange (reset))
        return VI_ERROR_PARAMETER3;

    /*- Open instrument session ---------------------------------------------*/

    if ((AVR3271_status = viOpenDefaultRM (&rmSession)) < 0)
        return AVR3271_status;

    if ((AVR3271_status = viOpen (rmSession, resourceName, VI_NULL,VI_NULL, instrSession)) < 0) {
        viClose (rmSession);
        return AVR3271_status;
    }

    /*- Configure VISA Formatted I/O ----------------------------------------*/
    cable = resourceName[0]; /* GPIB or RS-232 cable */
    
    if (cable=='G') { /*GPIB cable*/

        if ((AVR3271_status = viSetAttribute (*instrSession, VI_ATTR_TMO_VALUE, 10000)) < 0)
            return AVR3271_initCleanUp (rmSession, instrSession, AVR3271_status);
            
        if ((AVR3271_status = viSetBuf (*instrSession, VI_READ_BUF|VI_WRITE_BUF, 4000)) < 0)
            return AVR3271_initCleanUp (rmSession, instrSession, AVR3271_status);
            
        if ((AVR3271_status = viSetAttribute (*instrSession, VI_ATTR_WR_BUF_OPER_MODE,
                                             VI_FLUSH_ON_ACCESS))   < 0)
            return AVR3271_initCleanUp (rmSession, instrSession, AVR3271_status);
            
        if ((AVR3271_status = viSetAttribute (*instrSession, VI_ATTR_RD_BUF_OPER_MODE,
                                             VI_FLUSH_ON_ACCESS))   < 0)
            return AVR3271_initCleanUp (rmSession, instrSession, AVR3271_status);
        
        Fmt (Buffer,"%s<%s",s_init[0]);     
        if ((AVR3271_status = viWrite(*instrSession,Buffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3271_initCleanUp (rmSession, instrSession, AVR3271_status);
            
    }
    
    else { /*RS-232 cable*/
        /* Serial Interface initialized to 8 databits, 1 stopbit, no parity, */
        /* baud rate = 19200, flow control = XonXoff and timeout = 20000 ms  */
        
        if ((AVR3271_status = viSetAttribute (*instrSession, VI_ATTR_ASRL_DATA_BITS, 8))<0)
            return AVR3271_initCleanUp (rmSession, instrSession, AVR3271_status);
    
        if ((AVR3271_status = viSetAttribute (*instrSession, VI_ATTR_ASRL_STOP_BITS,VI_ASRL_STOP_ONE))<0)
            return AVR3271_initCleanUp (rmSession, instrSession, AVR3271_status);
            
        if ((AVR3271_status = viSetAttribute (*instrSession, VI_ATTR_ASRL_PARITY, VI_ASRL_PAR_NONE))<0)
            return AVR3271_status;
        
        if ((AVR3271_status = viSetAttribute (*instrSession, VI_ATTR_ASRL_BAUD, 19200))<0)
            return AVR3271_status;
        
        if ((AVR3271_status = viSetAttribute (*instrSession, VI_ATTR_ASRL_FLOW_CNTRL,VI_ASRL_FLOW_XON_XOFF))<0)
            return AVR3271_status;                 
        
        if ((AVR3271_status = viSetAttribute (*instrSession, VI_ATTR_TMO_VALUE, 20000)) < 0)
            return AVR3271_status;                     
   
        if ((AVR3271_status = viSetBuf (*instrSession, VI_ASRL_IN_BUF|VI_ASRL_OUT_BUF, 512)) < 0)
            return AVR3271_initCleanUp (rmSession, instrSession, AVR3271_status);
   
        if ((AVR3271_status = viSetBuf (*instrSession, VI_READ_BUF|VI_WRITE_BUF, 4000)) < 0)
            return AVR3271_initCleanUp (rmSession, instrSession, AVR3271_status);
   
        if ((AVR3271_status = viSetAttribute (*instrSession, VI_ATTR_WR_BUF_OPER_MODE,VI_FLUSH_ON_ACCESS))<0)
            return AVR3271_initCleanUp (rmSession, instrSession, AVR3271_status);
    
        if ((AVR3271_status = viSetAttribute (*instrSession, VI_ATTR_RD_BUF_OPER_MODE,VI_FLUSH_ON_ACCESS)) < 0)
            return AVR3271_initCleanUp (rmSession, instrSession, AVR3271_status);
 
        if ((AVR3271_status = viSetAttribute (*instrSession, VI_ATTR_TERMCHAR, '\n')) < 0)
            return AVR3271_initCleanUp (rmSession, instrSession, AVR3271_status);
 
        if ((AVR3271_status = viSetAttribute (*instrSession, VI_ATTR_ASRL_END_IN,VI_ASRL_END_TERMCHAR)) < 0)
            return AVR3271_initCleanUp (rmSession, instrSession, AVR3271_status);
   
        if ((AVR3271_status = viSetAttribute (*instrSession, VI_ATTR_ASRL_END_OUT,VI_ASRL_END_NONE)) < 0)
            return AVR3271_initCleanUp (rmSession, instrSession, AVR3271_status);       
            
    }       

    /*- Identification Query -------------------------------------------------*/
    
    if (IDquery) {  /* identification executed */
    
        Fmt (Buffer,"%s<%s,%s\r\n",s_init[1],s_init[2]);     
        if ((AVR3271_status = viWrite(*instrSession,Buffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3271_initCleanUp (rmSession, instrSession, AVR3271_status);
    
        if ((AVR3271_status = viRead (*instrSession, Buffer, BUFFER_SIZE, &retCnt)) < 0)
            return AVR3271_initCleanUp (rmSession, instrSession, AVR3271_status);
         
        if (Buffer[0] != 'R')  
            return AVR3271_initCleanUp (rmSession, instrSession, VI_ERROR_FAIL_ID_QUERY);
         
        interim[0] = Buffer[1];
        interim[1] = Buffer[2];
        interim[2] = Buffer[3];
        interim[3] = Buffer[4];
        interim[4] = Buffer[5];
      
        if (Scan(interim, "%s>%d[b4]", &AVR3271_instrType)!=1) {
            AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
            return AVR3271_status;
        } 
        
    }
    
    else { /* no identification executed */
        
        Fmt (Buffer,"%s<%s\r\n",s_init[1]);     
        if ((AVR3271_status = viWrite(*instrSession,Buffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3271_initCleanUp (rmSession, instrSession, AVR3271_status);
        
        AVR3271_instrType = 3271 ;
        
    }   
     
   
    /*- Reset instrument ----------------------------------------------------*/
    if (reset) {
        if ((AVR3271_status = AVR3271_initReset (*instrSession,AVR3271_instrType)) < 0)
            return AVR3271_initCleanUp (rmSession, instrSession, AVR3271_status);
    }      
    else { /*- Send Default Instrument Setup ---------------------------------*/ 
        if ((AVR3271_status = AVR3271_defaultInstrSetup (*instrSession,AVR3271_instrType)) < 0)
            return AVR3271_initCleanUp (rmSession, instrSession, AVR3271_status);
    }
    
    return AVR3271_status;
}

/*===========================================================================*/
/* Function : configure and read                                             */
/* Purpose : application function                                            */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_configRead(ViSession instrID,ViBoolean trace,ViInt16 peakSearch,
                                    ViPInt16 traceAccuracy,ViInt16 _VI_FAR traceArray[],
                                    ViPReal64 sweepTime,ViChar _VI_FAR sweepUnit[],
                                    ViPReal64 MKRNposition,ViPReal64 MKRNlevel)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    AVR3271_ptrFeatures featuresPtr;
    ViInt16 status,control;
    ViReal64 sweepValue;
    
    if ((AVR3271_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3271_status;
       
    if (AVR3271_invalidViBooleanRange (trace))             
        return VI_ERROR_PARAMETER2;
        
    if (AVR3271_invalidViInt16Range (peakSearch,0,5))             
        return VI_ERROR_PARAMETER3;
        
    if ((AVR3271_status = AVR3271_Read_TraceAccuracy (instrID, traceAccuracy))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = AVR3271_Read_TraceData (instrID, trace,1,traceArray))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = AVR3271_Search_Peak (instrID, peakSearch)) <0)
        return AVR3271_status;   
       
    if ((AVR3271_status = AVR3271_Read_SweepTime (instrID, &sweepValue, &control))<0)
        return AVR3271_status;
       
    if ((AVR3271_status = AVR3271_Convert_Time (instrID, sweepValue, sweepTime, sweepUnit))<0)
        return AVR3271_status;
       
    if ((AVR3271_status = AVR3271_Read_NormalMarker (instrID, MKRNposition, MKRNlevel))<0)
        return AVR3271_status;   
       
    return AVR3271_status;
} 

/*===========================================================================*/
/* Function: Set Frequency                                                   */           
/* Purpose:  sets up center, start or stop frequency.                        */           
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3271_Set_Freq (ViSession instrID, ViInt16 freqType,
                                                       ViReal64 freqValue)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVR3271_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;

    if ((AVR3271_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3271_status;
        
    if (AVR3271_invalidViInt16Range (freqType, 0, 2))             
        return VI_ERROR_PARAMETER2;
        
    if (AVR3271_invalidViReal64Range (freqValue, featuresPtr->frequencyLow[freqType],
                                                 featuresPtr->frequencyHigh[freqType]))             
        return VI_ERROR_PARAMETER3;
    
    Fmt (wrtBuffer,"%s<%s%fHZ\r\n",s_freqType[freqType],freqValue);     
    if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3271_status;
                                  
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Set  Frequency Span                                             */           
/* Purpose:  sets up the frequency span.                                     */           
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3271_Set_Span (ViSession instrID, ViInt16 spanType,
                                                       ViReal64 spanValue)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVR3271_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    
    if ((AVR3271_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3271_status;

    if (AVR3271_invalidViInt16Range (spanType, 0, 2))             
        return VI_ERROR_PARAMETER2;                         
       
    if (spanType==0) {  /* Manual Span Setting */ 
        if (AVR3271_invalidViReal64Range (spanValue, featuresPtr->spanLow, 
                                                     featuresPtr->spanHigh))             
            return VI_ERROR_PARAMETER3;
        Fmt (wrtBuffer,"%s<%s%fHZ\r\n",s_spanType[spanType],spanValue);     
        if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3271_status;
    }
    else { /*full span or zero span*/
        Fmt (wrtBuffer,"%s<%s\r\n",s_spanType[spanType]);     
        if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3271_status;
    }
    
    if ((spanType==2)||(spanValue==0)) featuresPtr->span=0;  /*span null*/
    else featuresPtr->span=1; /*span not null*/
     
    return AVR3271_status;
}                         

/*===========================================================================*/
/* Function: Set Frequency Offset                                            */           
/* Purpose:  sets up frequency offset and activates frequency offset         */           
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3271_Set_FreqOffset (ViSession instrID, ViBoolean activation,
                                                             ViReal64 freqOffset)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVR3271_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    
    if ((AVR3271_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3271_status;
       
    if (AVR3271_invalidViBooleanRange (activation))             
        return VI_ERROR_PARAMETER2;   

    if (activation == VI_ON) { /*activation*/
        if (AVR3271_invalidViReal64Range (freqOffset,featuresPtr->freqOffsetMin,
                                                     featuresPtr->freqOffsetMax))             
            return VI_ERROR_PARAMETER3;
        Fmt (wrtBuffer,"%s<%s%fHZ\r\n", s_freqOffset[activation], freqOffset);     
        if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3271_status;
    }
    else { /*desactivation*/
        Fmt (wrtBuffer,"%s<%s\r\n",s_freqOffset[activation]);     
        if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3271_status;
    }
     
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Set Frequency Step Size                                         */           
/* Purpose:  sets up and activates or deactivates Frequency Step Size        */           
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3271_Set_FreqStepSize (ViSession instrID, ViBoolean controlType,
                                                               ViReal64 freqStepSize)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVR3271_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    
    if ((AVR3271_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3271_status;

    if (AVR3271_invalidViBooleanRange (controlType))             
        return VI_ERROR_PARAMETER2;
       
    if (controlType == VI_ON) { /*step size chosen by the analyzer*/ 
        Fmt (wrtBuffer,"%s<%s\r\n", s_freqStepSize[controlType]);     
        if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3271_status;
    }
    else { /*step size chosen by the user*/   
        if (AVR3271_invalidViReal64Range (freqStepSize,featuresPtr->spanLow,
                                                       featuresPtr->spanHigh))             
            return VI_ERROR_PARAMETER3;   
        Fmt (wrtBuffer,"%s<%s%fHZ\r\n",s_freqStepSize[controlType],freqStepSize);     
        if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3271_status;
    }
     
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Set Frequency Counter                                           */
/* Purpose:  Sets the counter ON / OFF or sets the counter resolution        */
/*===========================================================================*/                               
ViStatus _VI_FUNC AVR3271_Set_FreqCounter (ViSession instrID, ViBoolean activation,
                                                              ViInt16 resolution)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVR3271_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    
    if (AVR3271_invalidViBooleanRange (activation))             
        return VI_ERROR_PARAMETER2;
       
    if (AVR3271_invalidViInt16Range (resolution,0,3))             
        return VI_ERROR_PARAMETER3;
       
    if (activation == VI_ON) {
        Fmt (wrtBuffer,"%s<%s\r\n", s_counterResolution[resolution]);     
        if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3271_status; 
    }

    Fmt (wrtBuffer,"%s<%s\r\n", s_counterONOFF[activation]);     
    if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3271_status;

    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Set Reference Level                                             */           
/* Purpose:  sets up the reference level (Y axis).                           */           
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3271_Set_RefLevel (ViSession instrID, ViReal64 refLevel)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    AVR3271_ptrFeatures featuresPtr;
    ViInt16 unit=0;
    ViUInt32 retCount=0;
    
    if ((AVR3271_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3271_status;
    
    /* sets up the reference level */
    
    if (AVR3271_invalidViReal64Range (refLevel,featuresPtr->refLevelMin[featuresPtr->refUnit],
                                               featuresPtr->refLevelMax[featuresPtr->refUnit]))             
        return VI_ERROR_PARAMETER2;
        
    Fmt (Buffer,"%s<%s%f%s\r\n",s_refLevel[0],refLevel,s_levelUnit[featuresPtr->refUnit]);     
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3271_status;
    
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Set Reference Unit                                              */           
/* Purpose:  Set up the unit of the reference level.                         */           
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3271_Set_RefUnit (ViSession instrID, ViInt16 refUnit)
{   
    ViStatus AVR3271_status = VI_SUCCESS;  
    ViByte wrtBuffer[BUFFER_SIZE];
    AVR3271_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    
    if ((AVR3271_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3271_status;
    
    if (AVR3271_invalidViInt16Range (refUnit,0,7))             
        return VI_ERROR_PARAMETER2;
    
    featuresPtr->refUnit = refUnit;    
    
    Fmt (wrtBuffer,"%s<%s\r\n",s_refUnit[refUnit]);     
    if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3271_status;
       
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Set Scale                                                       */           
/* Purpose:  Set up the scale of the Y axis .                                */           
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3271_Set_Scale (ViSession instrID, ViInt16 scale)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVR3271_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    
    if ((AVR3271_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3271_status;

    if (AVR3271_invalidViInt16Range (scale,0,10))             
        return VI_ERROR_PARAMETER2;
        
    Fmt (wrtBuffer,"%s<%s\r\n",s_yScale[scale]);     
    if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3271_status;
       
    featuresPtr->scale = scale;   
                                  
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Set Reference Level Offset                                      */           
/* Purpose:  Set up and activates the reference level offset.                */           
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3271_Set_RefOffset (ViSession instrID, ViBoolean activation,
                                                            ViReal64 refLevelOffset)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVR3271_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    
    if ((AVR3271_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3271_status;
       
    if (AVR3271_invalidViBooleanRange (activation))             
        return VI_ERROR_PARAMETER2;   

    if (activation == VI_ON) {  /* activation */
        if (AVR3271_invalidViReal64Range (refLevelOffset,featuresPtr->refLevelOffsetMin,
                                                         featuresPtr->refLevelOffsetMax))             
            return VI_ERROR_PARAMETER3;
        Fmt (wrtBuffer,"%s<%s%fDB\r\n", s_refLevelOffset[activation], refLevelOffset);     
        if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3271_status;
    }
    else { /* desactivation */
        Fmt (wrtBuffer,"%s<%s\r\n",s_refLevelOffset[activation]);     
        if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3271_status;
    }
    
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Set Input Attenuation                                           */           
/* Purpose:  Set up the Input attenuation.                                   */           
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3271_Set_Att (ViSession instrID, ViBoolean controlType,
                                                      ViReal64 inputAttValue)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVR3271_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    
    if ((AVR3271_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3271_status;
       
    if (AVR3271_invalidViBooleanRange (controlType))             
        return VI_ERROR_PARAMETER2;   

    if (controlType == VI_ON) { /* attenuation chosen by the analyzer */ 
        Fmt (wrtBuffer,"%s<%s\r\n", s_att[controlType]);     
        if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
             return AVR3271_status;
    }
    else { /* attenuation chosen by the user */
        if (AVR3271_invalidViReal64Range (inputAttValue, featuresPtr->inputAttMin,
                                                         featuresPtr->inputAttMax))             
            return VI_ERROR_PARAMETER3; 
        Fmt (wrtBuffer,"%s<%s%fDB\r\n",s_att[controlType],inputAttValue);     
        if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3271_status;
    }
     
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Set Minimum Input Attenuation                                   */           
/* Purpose:  Set up the minimum attenuation that can be set                  */           
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3271_Set_MinATT (ViSession instrID, ViBoolean controlType,
                                                         ViReal64 minInputAttValue)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVR3271_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    
    if ((AVR3271_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3271_status;
       
    if (AVR3271_invalidViBooleanRange (controlType))             
        return VI_ERROR_PARAMETER2;   

    if (controlType == VI_ON) { /* minimum attenuation chosen by the analyzer */
        Fmt (wrtBuffer,"%s<%s", s_attMin[controlType]);     
        if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3271_status;
        featuresPtr->minAttControl = 1;    
    }
    else { /* minimum attenuation chosen by the user */   
    	if (AVR3271_invalidViReal64Range (minInputAttValue, featuresPtr->inputAttMin,
                                                     	    featuresPtr->inputAttMax))             
        	return VI_ERROR_PARAMETER3;
        Fmt (wrtBuffer,"%s<%s%f",s_attMin[controlType],minInputAttValue);     
        if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3271_status;
        featuresPtr->minAttControl = 0;   
    }
     
    return AVR3271_status;
}     

/*===========================================================================*/
/* Function: Set High Sence Mode                                             */           
/* Purpose:  Set up the High Sensitive Mode.                                 */           
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3271_Set_HighSenceMode (ViSession instrID, ViBoolean mode)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVR3271_invalidViBooleanRange (mode))             
        return VI_ERROR_PARAMETER2;   

    Fmt (wrtBuffer,"%s<%s\r\n", s_highsence[mode]);     
    if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
    	return AVR3271_status;
    
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Set RBW                                                         */           
/* Purpose:  Set up the Resolution Bandwidth.                                */
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3271_Set_RBW (ViSession instrID, ViInt16 RBWsetting)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    if (AVR3271_invalidViInt16Range (RBWsetting,0,12))             
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuffer,"%s<%s\r\n",s_RBW[RBWsetting]);     
    if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3271_status;
       
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Set VBW                                                         */           
/* Purpose:  Set up the Video Bandwidth.                                     */
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3271_Set_VBW (ViSession instrID, ViInt16 VBWsetting)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    if (AVR3271_invalidViInt16Range (VBWsetting,0,14))             
        return VI_ERROR_PARAMETER2;
        
    Fmt (wrtBuffer,"%s<%s\r\n",s_VBW[VBWsetting]);     
    if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3271_status;    

    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Set All Auto Couple                                             */           
/* Purpose:  Sets all couple functions to automatic mode                     */
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3271_Set_All_Auto_Couple (ViSession instrID)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVR3271_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;

    Fmt (wrtBuffer,"%s<%s\r\n",s_All_Coupled[0]);     
    if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3271_status; 
    
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Set sweep time                                                  */           
/* Purpose:  Sets the sweep time (sec)                                       */
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3271_Set_SweepTime (ViSession instrID, ViBoolean controlType,
                                                            ViReal64 sweepTime)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVR3271_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;

    if ((AVR3271_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3271_status;
       
    if (AVR3271_invalidViBooleanRange (controlType))             
        return VI_ERROR_PARAMETER2;
        
    if (controlType == VI_ON) { /*sweep time is chosen by the analyzer*/  
        Fmt (wrtBuffer,"%s<%s\r\n",s_sweepTime[controlType]);     
        if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3271_status;  
    }
    else { /*sweep time is chosen by the user*/ 
        if (AVR3271_invalidViReal64Range (sweepTime,featuresPtr->sweepTimeMini[featuresPtr->span],
                                                    featuresPtr->sweepTimeMaxi[featuresPtr->span]))             
            return VI_ERROR_PARAMETER3;
        Fmt (wrtBuffer,"%s<%s%fSC\r\n",s_sweepTime[controlType],sweepTime);     
        if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3271_status;  
    }
    
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Set sweep mode                                                  */           
/* Purpose:  Sets the sweep mode: continuous / single, window sweep on / off */
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3271_Set_SweepMode (ViSession instrID, ViBoolean sweepMode,
                                                            ViBoolean windowSweep)
{
    ViStatus AVR3271_status = VI_SUCCESS;  
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    if (AVR3271_invalidViBooleanRange (sweepMode))             
        return VI_ERROR_PARAMETER2;
       
    if (AVR3271_invalidViBooleanRange (windowSweep))             
        return VI_ERROR_PARAMETER3;
     
    Fmt (wrtBuffer,"%s<%s,%s\r\n",s_sweepMode[sweepMode],s_windowSweep[windowSweep]);     
    if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3271_status;
     
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Set trigger                                                     */           
/* Purpose:  Defines the trigger source (except TV trigger, the polarity of  */
/*           the video and external triggers, the levels at which the video  */
/*           and external trigger occurs                                     */
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3271_Set_Trigger(ViSession instrID, ViInt16 source,
                                                         ViBoolean polarity,
                                                         ViInt32 videoLevel)
{
    ViStatus AVR3271_status = VI_SUCCESS;  
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    if (AVR3271_invalidViInt16Range (source,0,3))             
        return VI_ERROR_PARAMETER2;
        
    if (AVR3271_invalidViBooleanRange (polarity))             
        return VI_ERROR_PARAMETER3;
        
    if (source<=1) { /* free run or line */
        Fmt (wrtBuffer,"%s<%s\r\n",s_trigger[source]);     
        if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3271_status; 
    }
    
    else if (source==2) { /* video trigger */
        if (AVR3271_invalidViInt16Range (videoLevel,0,100))             
            return VI_ERROR_PARAMETER4;
        Fmt (wrtBuffer,"%s<%s%d[b4]ENT\r\n",s_trigger[source],videoLevel);     
        if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3271_status;
    }
    
    else if (source==3) {  /* external trigger */
        Fmt (wrtBuffer,"%s<%s,%s\r\n",s_trigger[source],s_trigger[4+polarity]);     
        if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3271_status;
    }       

    return AVR3271_status;
}
 
/*===========================================================================*/
/* Function: Set trigger position                                            */           
/* Purpose:  Defines the position along the x-axis at which the trigger      */
/*           occurs                                                          */
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3271_Set_TriggerPosition(ViSession instrID, ViInt16 position)
{
    ViStatus AVR3271_status = VI_SUCCESS;  
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    if (AVR3271_invalidViInt16Range (position,0,100))             
        return VI_ERROR_PARAMETER2;
        
    Fmt (wrtBuffer,"%s<%s%d[b2]ENT\r\n",s_trigger[5],position);     
    if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3271_status; 
    
    return AVR3271_status;
} 

/*===========================================================================*/
/* Function: Set trigger TV                                                  */           
/* Purpose:  Defines the trigger TV source,the polarity of this trigger      */
/*           and the line number at which the trigger occurs                 */
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3271_Set_TriggerTV(ViSession instrID, ViInt16 triggerSource,
                                                          ViBoolean triggerPolarity,
                                                          ViInt16 lineNumber)
{
    ViStatus AVR3271_status = VI_SUCCESS;  
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    if (AVR3271_invalidViInt16Range (triggerSource,0,2))             
        return VI_ERROR_PARAMETER2;
        
    if (AVR3271_invalidViBooleanRange (triggerPolarity))             
        return VI_ERROR_PARAMETER3;
        
    if (triggerSource != 0) { /* TV-H */
        if (AVR3271_invalidViInt16Range (lineNumber,1,625))             
            return VI_ERROR_PARAMETER4;
        Fmt (wrtBuffer,"%s<%s,%s,%s%d[b2]ENT\r\n",s_triggerTV[triggerSource],
                                                  s_triggerTV[3+triggerPolarity],
                                                  s_triggerTV[5],lineNumber);     
    }
    else { /* TV-V */
        Fmt (wrtBuffer,"%s<%s,%s\r\n",s_triggerTV[triggerSource],
                                      s_triggerTV[3+triggerPolarity]);
    }
                                                  
    if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3271_status; 

    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Set Trace A Mode                                                */           
/* Purpose:  Defines Trace A mode (Write, View, Max Hold ...)                */
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3271_Set_TraceAMode (ViSession instrID, ViInt16 traceAMode,
                                                             ViInt16 averageCount,
                                                             ViBoolean averageMode)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVR3271_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;

    if ((AVR3271_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3271_status;

    if (AVR3271_invalidViInt16Range (traceAMode,0,5))             
        return VI_ERROR_PARAMETER2;
       
    if (traceAMode != 5) { /* set trace mode except trace average */ 
        Fmt (wrtBuffer,"%s<%s\r\n",s_traceAMode[traceAMode]);   
        if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return AVR3271_status;
    }
    else { /* set trace average: some parameters are needed */
        if (AVR3271_invalidViInt16Range (averageCount,featuresPtr->averageCountMini,
                                                      featuresPtr->averageCountMaxi))             
            return VI_ERROR_PARAMETER3;
        if (AVR3271_invalidViBooleanRange (averageMode))             
            return VI_ERROR_PARAMETER4;
        Fmt (wrtBuffer,"%s<%s,%s%d[b2]ENT\r\n",s_averageMode[averageMode],
                                               s_traceAMode[traceAMode],averageCount);   
        if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return AVR3271_status;
    }
    
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Set Trace B Mode                                                */           
/* Purpose:  Defines Trace B mode : Store, View or Blank                     */
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3271_Set_TraceBMode (ViSession instrID, ViInt16 traceBMode,
                                                             ViInt16 averageCount,
                                                             ViBoolean averageMode)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVR3271_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;

    if ((AVR3271_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3271_status;

    if (AVR3271_invalidViInt16Range (traceBMode,0,5))             
        return VI_ERROR_PARAMETER2;
       
    if (traceBMode != 5) { /* set trace mode except trace average */ 
        Fmt (wrtBuffer,"%s<%s\r\n",s_traceBMode[traceBMode]);   
        if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return AVR3271_status;
    }
    else { /* set trace average: some parameters are needed */
        if (AVR3271_invalidViInt16Range (averageCount,featuresPtr->averageCountMini,
                                                      featuresPtr->averageCountMaxi))             
            return VI_ERROR_PARAMETER3;
        if (AVR3271_invalidViBooleanRange (averageMode))             
            return VI_ERROR_PARAMETER4;
        Fmt (wrtBuffer,"%s<%s,%s%d[b2]ENT\r\n",s_averageMode[averageMode],
                                               s_traceBMode[traceBMode],averageCount);   
        if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return AVR3271_status;
    }
    
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Set Detector Mode                                               */           
/* Purpose:  Specifies detection mode: posi-nega, posi-peak...               */
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3271_Set_TraceDetector (ViSession instrID, ViInt16 detectorMode) 
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    if (AVR3271_invalidViInt16Range (detectorMode,0,3))             
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuffer,"%s<%s\r\n",s_detectorMode[detectorMode]);   
    if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    return AVR3271_status;                                          
}

/*===========================================================================*/
/* Function: Set Limit Line                                                  */
/* Purpose:  Defines or clears a whole limit line table                      */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Set_LimitLine (ViSession instrID, ViBoolean limitLine,
                                                           ViString LL_FreqTimeTable,
                                                           ViString LL_LevelTable)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    ViReal64 freqtime[51];
    ViReal64 level[51];
    ViInt16 i=0;
    AVR3271_ptrFeatures featuresPtr;
    
    if ((AVR3271_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3271_status;
    
    if (AVR3271_invalidViBooleanRange (limitLine))             
        return VI_ERROR_PARAMETER2;
    
    for (i=0; i<51; i++) {
        freqtime[i]=0;
        level[i]=0;
    }
    
    Scan(LL_FreqTimeTable,"%s>%51f[x]",freqtime);
    Scan(LL_LevelTable,"%s>%51f[x]",level);
    
    Fmt (Buffer,"%s<%s,%s\r\n",s_LLType[featuresPtr->span],s_deleteLL[limitLine]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    i=0;
    while ( (i<=48)&&((freqtime[i]!=0)||(freqtime[i+1]!=0)||(freqtime[i+2]!=0)) ) {
        Fmt (Buffer,"%s<%s%f%s,%fDB\r\n",s_setLL[limitLine],freqtime[i],
                                         s_positionUnit[featuresPtr->span],level[i]);   
        if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
            return AVR3271_status;
        i++;   
    }   
     
    return AVR3271_status; 
}

/*===========================================================================*/
/* Function: Activate Limit Line                                             */
/* Purpose:  Show Limit Line 1 and/or Limit Line 2                           */
/*           Choose between absolute or relative along the two axis          */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Activate_LimitLine (ViSession instrID,ViBoolean LL1Status,
                                                               ViBoolean LL2Status,
                                                               ViBoolean levelXaxis,
                                                               ViBoolean levelYaxis,
                                                               ViBoolean passfail)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    if (AVR3271_invalidViBooleanRange (LL1Status))             
        return VI_ERROR_PARAMETER2;
        
    if (AVR3271_invalidViBooleanRange (LL2Status))             
        return VI_ERROR_PARAMETER3;
        
    if (AVR3271_invalidViBooleanRange (levelXaxis))             
        return VI_ERROR_PARAMETER4;
    
    if (AVR3271_invalidViBooleanRange (levelYaxis))             
        return VI_ERROR_PARAMETER5;
        
    if (AVR3271_invalidViBooleanRange (passfail))             
        return VI_ERROR_PARAMETER6;
        
        
    
    Fmt (wrtBuffer,"%s<%s,%s,%s,%s,%s\r\n",s_showLL1[LL1Status],s_showLL2[LL2Status],
                                           s_levelX[levelXaxis],s_levelY[levelYaxis],
                                           s_LLpassfail[passfail]);   
    if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    return AVR3271_status;                                          
}

/*===========================================================================*/
/* Function: Set Display Line                                                */           
/* Purpose:  Specifies display line position and show or hide it             */
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3271_Set_DisplayLine (ViSession instrID,ViBoolean showDL,
                                                            ViReal64 DL_Level)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVR3271_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;

    if ((AVR3271_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3271_status;

    if (AVR3271_invalidViBooleanRange (showDL))             
        return VI_ERROR_PARAMETER2;
    
    if (showDL == VI_ON) { /* activation */
        if (AVR3271_invalidViReal64Range (DL_Level,featuresPtr->DL_LevelMin,
                                                  featuresPtr->DL_LevelMax))             
            return VI_ERROR_PARAMETER3;
        Fmt (wrtBuffer,"%s<%s %f%s\r\n",s_showDL[showDL],DL_Level,
                                        s_levelUnit[featuresPtr->refUnit]);   
        if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return AVR3271_status;
    }
    else {  /* desactivation */
        Fmt (wrtBuffer,"%s<%s\r\n",s_showDL[showDL]);   
        if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return AVR3271_status;
    } 
     
    return AVR3271_status;                                          
}

/*===========================================================================*/
/* Function: Set measurement Window                                          */
/* Purpose:  Sets center position, Window width and shows or hides           */
/*           mesurement window                                               */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Set_MeasurementWindow (ViSession instrID,ViBoolean windowStatus,
                                                                  ViReal64 Xposition,
                                                                  ViReal64 Xwidth,
                                                                  ViReal64 Yposition,
                                                                  ViReal64 Ywidth)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViReal64 levelmax,levelmin;
    ViUInt32 retCount=0;
    AVR3271_ptrFeatures featuresPtr;
    
    if ((AVR3271_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3271_status;
    
    if (AVR3271_invalidViBooleanRange (windowStatus))             
        return VI_ERROR_PARAMETER2;
        
    if (windowStatus == VI_OFF) {
        Fmt (wrtBuffer,"%s<%s\r\n",s_showWindow[windowStatus]);   
        if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return AVR3271_status;
    }
    else {
        levelmax=Yposition + 0.5*Ywidth;
        levelmin=Yposition - 0.5*Ywidth;
        Fmt (wrtBuffer,"%s<%s%f%s,%s%f%s,%s%f%s,%s%f%s,%s\r\n",
             s_positionWindow[0],Xposition,s_positionUnit[featuresPtr->span],
             s_positionWindow[1],Xwidth,s_positionUnit[featuresPtr->span],
             s_positionWindow[2],levelmax,s_levelUnit[featuresPtr->refUnit],
             s_positionWindow[3],levelmin,s_levelUnit[featuresPtr->refUnit],
             s_showWindow[windowStatus]);   
        if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return AVR3271_status;
    } 
    
    return AVR3271_status;
}  

/*===========================================================================*/
/* Function: set level correction                                            */
/* Purpose:  Activates and sets or deactivates level correction function     */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Set_LevelCorrection (ViSession instrID, ViBoolean LCstatus,
                                                                 ViString LC_FreqTable,
                                                                 ViString LC_LevelTable)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    ViReal64 freq[27];
    ViReal64 level[27];
    ViInt16 i=0;
    
    if (AVR3271_invalidViBooleanRange (LCstatus))             
        return VI_ERROR_PARAMETER2;
        
    if (LCstatus == VI_ON) {
      
        for (i=0; i<27; i++) {
            freq[i]=0;
            level[i]=0;
        }
    
        Scan(LC_FreqTable,"%s>%27f[x]",freq);
        Scan(LC_LevelTable,"%s>%27f[x]",level);
    
        Fmt (wrtBuffer,"%s<%s\r\n",s_workLC[0]);   
        if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return AVR3271_status;
         
        i=0;
        while ( (i<=24)&&((freq[i]!=0)||(freq[i+1]!=0)||(freq[i+2]!=0)) ) {
            Fmt (wrtBuffer,"%s<%s%fHZ,%fDB\r\n",s_workLC[1],freq[i],level[i]);   
            if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
                return AVR3271_status;
            i++;   
        }
    }   
       
    Fmt (wrtBuffer,"%s<%s\r\n",s_showLC[LCstatus]);   
    if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status; 
     
    return AVR3271_status; 
}

/*===========================================================================*/
/* Function: Set Normal Marker                                               */
/* Purpose:  sets the normal marker position                                 */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Set_NormalMarker (ViSession instrID,ViReal64 position)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    AVR3271_ptrFeatures featuresPtr;
    
    if ((AVR3271_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3271_status;
    
    Fmt (wrtBuffer,"%s<%s%f%s\r\n",s_mkrNormal[0],position,s_positionUnit[featuresPtr->span]);   
    if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;   
    
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Set Delta Marker                                                */
/* Purpose:  Sets the delta marker position                                  */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Set_DeltaMarker (ViSession instrID, ViReal64 position)
{
    ViStatus AVR3271_status = VI_SUCCESS; 
    ViByte wrtBuffer[BUFFER_SIZE];      
    ViUInt32 retCount=0;
    ViInt16 status;
    ViChar message[40];
    ViReal64 MKRNpos, MKRDpos, level;
    AVR3271_ptrFeatures featuresPtr;
    
    if ((AVR3271_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3271_status;
        
    AVR3271_Read_MarkerStatus (instrID, &status, message);
    
    if (status == 2) {  /* normal and delta markers are always active */
        
        AVR3271_Read_NormalMarker (instrID, &MKRNpos, &level);
        AVR3271_Read_DeltaMarker (instrID, &MKRDpos, &level);
        
        Fmt (wrtBuffer,"%s<%s%f%s\r\n",s_mkrDelta[0],0.0,s_positionUnit[featuresPtr->span]);   
        if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return AVR3271_status; 
        
        Fmt (wrtBuffer,"%s<%s%f%s\r\n",s_mkrNormal[0],MKRDpos,s_positionUnit[featuresPtr->span]);   
        if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return AVR3271_status;
            
        Fmt (wrtBuffer,"%s<%s%f%s\r\n",s_mkrDelta[0],MKRNpos,s_positionUnit[featuresPtr->span]);   
        if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return AVR3271_status;
    }       
    
    Fmt (wrtBuffer,"%s<%s%f%s\r\n",s_mkrDelta[0],position,s_positionUnit[featuresPtr->span]);   
    if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Set delta marker options                                        */
/* Purpose:  Sets the display mode and activates or deactivates delta        */
/*           marker fixed and 1/delta marker                                 */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Set_DeltaMarkerOptions (ViSession instrID,ViBoolean divided,
                                                                    ViBoolean fixed)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];      
    ViUInt32 retCount=0;

    if (AVR3271_invalidViBooleanRange (divided))             
        return VI_ERROR_PARAMETER2;
       
    if (AVR3271_invalidViBooleanRange (fixed))             
        return VI_ERROR_PARAMETER3;
       
    Fmt (wrtBuffer,"%s<%s,%s\r\n",s_1_deltaMkr[divided],s_fixedMkrDelta[fixed]);   
    if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
          
    return AVR3271_status;
}

/*===========================================================================*/
/* Function : Set Marker Trace                                               */
/* Purpose : chooses the trace on which the markers will be active           */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Set_MarkerTrace (ViSession instrID, ViBoolean markerTrace)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVR3271_invalidViBooleanRange (markerTrace))             
        return VI_ERROR_PARAMETER2;
    
    Fmt (Buffer,"%s<%s\r\n",s_mkrTrace[markerTrace]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
       
    return AVR3271_status;
}    
    
/*===========================================================================*/
/* Function: Activate Multimarkers                                           */
/* Purpose:  activates the multimarkers                                      */
/*===========================================================================*/                                  
ViStatus _VI_FUNC AVR3271_Activate_Multimarkers (ViSession instrID)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_multiMkrStatus[0]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    return AVR3271_status;                              
} 

/*===========================================================================*/
/* Function: Set Multimarkers                                                */
/* Purpose:  sets the Multi_Markers (frequency or time)                      */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Set_Multimarkers (ViSession instrID, ViString multiMkrStatus,
                                                               ViString multiMkrPositions)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    ViReal64 freqtime[8];
    ViInt16 i=0;
    AVR3271_ptrFeatures featuresPtr;
    
    if ((AVR3271_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3271_status;
    
    for (i=0; i<8; i++) {
        freqtime[i]=0;
    }
    
    Scan(multiMkrStatus,"%s>%8d[b2x]",featuresPtr->MKRMstatus);
    Scan(multiMkrPositions,"%s>%8f[x]",freqtime);
    
    for (i=0; i<8; i++) {
        if (featuresPtr->MKRMstatus[i]!= 0) {
            Fmt (wrtBuffer,"%s<%s%f%s\r\n",s_multiMkrON[i],freqtime[i],s_positionUnit[featuresPtr->span]);   
            if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
                return AVR3271_status;
        }
        else {
            Fmt (wrtBuffer,"%s<%s\r\n",s_multiMkrOFF[i]);   
            if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
                return AVR3271_status;
        }
    }
    
    return AVR3271_status; 
    
}

/*===========================================================================*/
/* Function: Set ActiveMultimarker                                           */
/* Purpose:  chooses the active Multi-Marker                                 */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Set_ActiveMultiMarker (ViSession instrID, ViInt16 index)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    ViInt16 status=0;
    ViReal64 MKRMpositions[8];
    ViReal64 MKRDposition;
    ViReal64 MKRMlevels[8];
    ViReal64 MKRDlevel;
    
    AVR3271_ptrFeatures featuresPtr;
    
    if ((AVR3271_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3271_status;
        
    if (AVR3271_invalidViInt16Range (index,1,8))             
        return VI_ERROR_PARAMETER2;
    
    AVR3271_Read_Multimarkers (instrID, MKRMpositions,&MKRDposition,
                                        MKRMlevels,&MKRDlevel);
                                       
    if (featuresPtr->MKRMstatus[index-1]!= 0) {                                   
        Fmt (Buffer,"%s<%s%f%s\r\n",s_multiMkrON[index-1],MKRMpositions[index-1],s_positionUnit[featuresPtr->span]);   
        if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
            return AVR3271_status; 
    }
    
    else {
        return VI_ERROR_MARKERINACTIVE;
    }   
    
    return AVR3271_status; 
}

/*===========================================================================*/
/* Function: Delete Markers                                                  */
/* Purpose:  deletes all the markers active (normal, delta, multimarkers)    */
/*===========================================================================*/                                  
ViStatus _VI_FUNC AVR3271_Delete_Markers (ViSession instrID)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_deleteMarker[0]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    return AVR3271_status;                              
} 

/*===========================================================================*/
/* Function: Set Peak Search Options                                         */
/* Purpose:  Sets the features of the peak search                            */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Set_PeakSearchOptions (ViSession instrID,ViBoolean continuousSearch,
                                                                  ViBoolean signalTrack,
                                                                  ViInt16 peakRange,
                                                                  ViReal64 deltaY)
{
    ViStatus AVR3271_status = VI_SUCCESS; 
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    AVR3271_ptrFeatures featuresPtr;
   
    if ((AVR3271_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3271_status;

    if (AVR3271_invalidViBooleanRange (continuousSearch))             
        return VI_ERROR_PARAMETER2;
       
    if (AVR3271_invalidViBooleanRange (signalTrack))             
        return VI_ERROR_PARAMETER3;
    
    if (AVR3271_invalidViInt16Range (peakRange,0,2))             
        return VI_ERROR_PARAMETER4;
       
    if (AVR3271_invalidViReal64Range(deltaY,featuresPtr->deltaYmin,
                                           featuresPtr->deltaYmax))
        return VI_ERROR_PARAMETER5;
        
    Fmt (wrtBuffer,"%s<%s,%s,%s,%s%fENT\r\n",s_continuousSearch[continuousSearch],
                                             s_signalTrack[signalTrack],
                                             s_peakRange[peakRange],s_peakRange[3],deltaY);   
    if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
       
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Search Peak                                                     */
/* Purpose:  set execute peak marker type                                    */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Search_Peak (ViSession instrID, ViInt16 typeSearch)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVR3271_invalidViInt16Range (typeSearch,0,5))             
        return VI_ERROR_PARAMETER2;
    
    Fmt (wrtBuffer,"%s<%s\r\n",s_peakSearch[typeSearch]);   
    if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
       
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: execute Peak to                                                 */
/* Purpose:  peak to setting                                                 */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Execute_PeakTo (ViSession instrID, ViBoolean peakTo)
{
    ViStatus AVR3271_status = VI_SUCCESS; 
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVR3271_invalidViBooleanRange (peakTo))             
        return VI_ERROR_PARAMETER2;
        
    Fmt (wrtBuffer,"%s<%s\r\n",s_peakTo[peakTo]);   
    if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
       
    return AVR3271_status;
}
 
/*===========================================================================*/
/* Function: Marker Delta to setting                                         */
/* Purpose:  marker delta to setting                                         */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Set_MarkerDeltaTo (ViSession instrID, ViInt16 deltaTo)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    if (AVR3271_invalidViInt16Range (deltaTo,0,3))             
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuffer,"%s<%s\r\n",s_markerDeltaTo[deltaTo]);   
    if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Take Sweep                                                      */
/* Purpose:  Sets sweep mode & starts sweeping                               */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Take_Sweep (ViSession instrID, ViInt16 startSweep)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
  
    if (AVR3271_invalidViInt16Range (startSweep,0,3))             
        return VI_ERROR_PARAMETER2;
     
    Fmt (wrtBuffer,"%s<%s\r\n",s_sweepMode[startSweep]);   
    if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;   
       
    return AVR3271_status;
}  

/*===========================================================================*/
/* Function: Execute Trace Math                                              */
/* Purpose:  Trace Math execution                                            */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Execute_TraceMath (ViSession instrID, ViInt16 traceMath)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVR3271_invalidViInt16Range (traceMath,0,4))             
        return VI_ERROR_PARAMETER2;
    
    Fmt (wrtBuffer,"%s<%s\r\n",s_traceMath[traceMath]);   
    if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;   
 
    return AVR3271_status;  
}

/*===========================================================================*/
/* Function: Normalizing Mode                                                */
/* Purpose:  Normalize Mode execution                                        */
/*===========================================================================*/ 
ViStatus _VI_FUNC AVR3271_NormalizingMode (ViSession instrID, ViInt16 mode)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVR3271_invalidViInt16Range (mode,0,3))             
        return VI_ERROR_PARAMETER2;
        
    Fmt (wrtBuffer,"%s<%s\r\n",s_NormMode[mode]);   
    if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;    
    
    return AVR3271_status;
}    

/*===========================================================================*/
/* Function: Start/Stop Trace Average                                        */
/* Purpose:  Starts stops trace average                                      */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Control_TraceAverage (ViSession instrID, ViInt16 traceAverage)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVR3271_invalidViInt16Range (traceAverage,0,3))             
        return VI_ERROR_PARAMETER2;
        
    Fmt (wrtBuffer,"%s<%s\r\n",s_traceAverage[traceAverage]);   
    if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;    
        
    return AVR3271_status;  
}

/*===========================================================================*/
/* Function: Read  Frequency Settings                                        */
/* Purpose:  Reads a frequency that you have chosen                          */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Read_FreqSettings (ViSession instrID, ViInt16 parameter,
                                                               ViPReal64 value)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVR3271_invalidViInt16Range (parameter,0,3))             
        return VI_ERROR_PARAMETER2;
    
    Fmt (Buffer,"%s<%s\r\n",s_readFreq[parameter]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVR3271_status;
       
    if (Scan(Buffer,"%s>%f",value)!=1) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }
     
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Read  Frequency Offset                                          */
/* Purpose:  Reads the frequency offset status and value                     */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Read_FreqOffset (ViSession instrID, ViPInt16 status,
                                                             ViPReal64 value)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_readFreq[4]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVR3271_status;
       
    if (Scan(Buffer,"%s>%d[b2]%f",status,value)!=2) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }
     
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Read Frequency Step Size                                        */
/* Purpose:  returns the frequency step size and says whether this parameter */
/*           is controled  by the user or by the analyzer                    */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Read_FreqStepSize (ViSession instrID, ViPReal64 value,
                                                               ViPInt16 controlType)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_readFreqStepSize[0]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVR3271_status;
       
    if (Scan(Buffer,"%s>%f",value)!=1) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }
     
    Fmt (Buffer,"%s<%s\r\n",s_readFreqStepSize[1]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVR3271_status;
          
    if (Scan(Buffer,"%s>%d[b2]", controlType)!=1) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }
      
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Read Frequency Counter                                          */
/* Purpose:  returns the frequency counter value and says whether the counter*/
/*           is active or inactive                                           */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Read_FreqCounter (ViSession instrID,ViPReal64 counterValue,
                                                             ViPInt16 counterStatus)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_readFreqCounter[0]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVR3271_status;
       
    if (Scan(Buffer,"%s>%d[b2]", counterStatus)!=1) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }
     
    Fmt (Buffer,"%s<%s\r\n",s_readFreqCounter[1]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVR3271_status;
          
    if (Scan(Buffer,"%s>%f", counterValue)!=1) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }
      
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Read  Reference Level                                           */
/* Purpose:  returns the value (without unit) of the reference level         */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Read_ReferenceLevel(ViSession instrID, ViPReal64 refLevelValue)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_readRefLevel[0]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVR3271_status;
       
    if (Scan(Buffer,"%s>%f", refLevelValue)!=1) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }
     
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Read Reference Level Offset                                     */
/* Purpose:  returns the value (unit is always dB) of the reference level    */
/* offset and says whether this offset is active or not                      */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Read_RefLevelOffset (ViSession instrID,ViPReal64 value,
                                                                ViPInt16 activation)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_readRefLevelOffset[0]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVR3271_status;
       
    if (Scan(Buffer,"%s>%d[b2x]%f",activation, value)!=2) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }
     
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Read Reference Unit                                             */
/* Purpose:  returns the reference unit                                      */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Read_RefUnit (ViSession instrID,ViPInt16 unitNumber,
                                                         ViChar _VI_FAR unitName[])
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_readRefUnit[0]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVR3271_status;
       
    if (Scan(Buffer,"%s>%d[b2]", unitNumber)!=1) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }
    
    Fmt(unitName,"%s<%s", s_readUnitName[*unitNumber]);  
     
    return AVR3271_status;
} 
                            
/*===========================================================================*/
/* Function: Read Scale                                                      */
/* Purpose:  reads the scale  status number and text                         */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Read_Scale (ViSession instrID, ViPInt16 status,
                                                        ViChar _VI_FAR scaleName[])
{
    ViStatus AVR3271_status = VI_SUCCESS;
    AVR3271_ptrFeatures featuresPtr;
    
    if ((AVR3271_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3271_status;
        
    *status = featuresPtr->scale;
    
    Fmt(scaleName,"%s<%s", s_readScale[*status]);
    
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Read Attenuation                                                */
/* Purpose:  returns the input attenuation or the minimum input attenuation  */
/*           and says whether this parameter is controled  by the user or by */
/*           the analyzer (only for the input attenuation)                   */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Read_Attenuation (ViSession instrID,ViPReal64 value,
                                                             ViPInt16 controlType)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_readAtt[0]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVR3271_status;
       
    if (Scan(Buffer,"%s>%f", value)!=1) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }
     
    Fmt (Buffer,"%s<%s\r\n",s_readAtt[1]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVR3271_status;
          
    if (Scan(Buffer,"%s>%d[b2]", controlType)!=1) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }
      
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Read RBW and VBW                                                */
/* Purpose:  reads the resolution or video bandwith and says whether this    */
/*           parameter is controlled by the analyzer or by the user          */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Read_RBWandVBW (ViSession instrID, ViBoolean parameter,
                                                            ViPReal64 value,
                                                            ViPInt16 controlType)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVR3271_invalidViBooleanRange (parameter))             
        return VI_ERROR_PARAMETER2;
    
    Fmt (Buffer,"%s<%s\r\n",s_readRBWandVBW[parameter]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVR3271_status;
       
    if (Scan(Buffer,"%s>%f", value)!=1) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }
     
    Fmt (Buffer,"%s<%s\r\n",s_readRBWandVBW[parameter+2]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVR3271_status;
        
    if (Scan(Buffer,"%s>%d[b2]", controlType)!=1) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    } 
     
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Read  Sweep Time                                                */
/* Purpose:  reads the sweep time value and says whether we are in auto or   */
/*           manual control                                                  */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Read_SweepTime (ViSession instrID, ViPReal64 timeValue,
                                                            ViPInt16 controlType)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_readSweepTime[0]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVR3271_status;
       
    if (Scan(Buffer,"%s>%f",timeValue)!=1) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }
     
    Fmt (Buffer,"%s<%s\r\n",s_readSweepTime[1]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVR3271_status;
       
    if (Scan(Buffer,"%s>%d[b2]", controlType)!=1) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    } 
     
    return AVR3271_status;
}                                                       

/*===========================================================================*/
/* Function: Read  Sweep Mode                                                */
/* Purpose:  returns the sweep Mode: Normal&Full, Single&Full, Normal&Window */
/*           or Single&Window                                                */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Read_SweepMode (ViSession instrID, ViPInt16 status,
                                                            ViChar _VI_FAR sweepMode[])
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_readSweepMode[0]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVR3271_status;
       
    if (Scan(Buffer,"%s>%d[b2]", status)!=1) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }
     
    if (*status==0)
        Fmt(sweepMode,"%s<%s",s_readSweepMode[1]);
    else if (*status==20)
        Fmt(sweepMode,"%s<%s",s_readSweepMode[2]);   
    else if (*status==1)
        Fmt(sweepMode,"%s<%s",s_readSweepMode[3]); 
    else if (*status==21)
        Fmt(sweepMode,"%s<%s",s_readSweepMode[4]);
     
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Read Trigger                                                    */
/* Purpose:  returns the informations about the trigger                      */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Read_Trigger (ViSession instrID, ViPInt16 triggerStatus,
                                                          ViChar _VI_FAR triggerName[],
                                                          ViPReal64 triggerFeature)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    ViInt16 index=0;
    
    /*searchs the trigger used*/
    
    Fmt (Buffer,"%s<%s\r\n",s_readTrigger[0]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVR3271_status;
       
    if (Scan(Buffer,"%s>%d[b2]", triggerStatus)!=1) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }
    
    if (*triggerStatus==0)
        Fmt(triggerName,"%s<%s",s_readTrigger[1]);
    else if (*triggerStatus==2) {
        Fmt(triggerName,"%s<%s",s_readTrigger[3]);
        index = 3;
    }    
    else if (*triggerStatus==3)
        Fmt(triggerName,"%s<%s",s_readTrigger[2]); 
    else if (*triggerStatus==4) {
        Fmt(triggerName,"%s<%s",s_readTrigger[4]);
        index = 4;
    }    
    else if (*triggerStatus==5) {
        Fmt(triggerName,"%s<%s",s_readTrigger[5]);    
        index = 5;
    }    
       
    Fmt (Buffer,"%s<%s\r\n",s_readTrigger[index+3]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVR3271_status;
        
    if (Scan(Buffer,"%s>%f", triggerFeature)!=1) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }   
     
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Read Trigger Position                                           */
/* Purpose:  returns the position of the trigger                             */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Read_TriggerPosition (ViSession instrID, ViPInt16 triggerPosition)
                                                          
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    ViInt16 index=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_readTrigger[9]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = viRead (instrID, Buffer, 10, &retCount)) < 0)
        return AVR3271_status;
       
    if (Scan(Buffer,"%s>%d[b2]", triggerPosition)!=1) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }
    
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Read Display Line                                               */
/* Purpose:  returns the position and the status of the Display Line         */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Read_DisplayLine(ViSession instrID, ViPReal64 level,
                                                             ViPInt16 status,
                                                             ViChar _VI_FAR message[])
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    AVR3271_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;

    if ((AVR3271_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3271_status;
       
    Fmt (Buffer,"%s<%s\r\n",s_readDisplayLine[0]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = viRead (instrID, Buffer, 20, &retCount)) < 0)
        return AVR3271_status;
       
    if (Scan(Buffer,"%s>%d[b2x]%f", status, level)!=2) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }
     
    Fmt(message,"%s<%s",s_textONOFF[*status]);
    
    return AVR3271_status;
}

/*===========================================================================*/
/* Function : Read Limit Line Options                                        */
/* Purpose : returns the options used with the limit lines                   */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Read_LimitLineOptions (ViSession instrID, ViBoolean parameter,
                                                                   ViPInt16 levelStatus,
                                                                   ViChar _VI_FAR levelMessage[])
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVR3271_invalidViBooleanRange (parameter))             
        return VI_ERROR_PARAMETER2;
       
    Fmt (Buffer,"%s<%s\r\n",s_readLLoptions[parameter]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVR3271_status;
       
    if (Scan(Buffer,"%s>%d[b2]", levelStatus)!=1) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }
     
    Fmt(levelMessage,"%s<%s",s_readLLmessage[*levelStatus]);
    
    return AVR3271_status;
}  

/*===========================================================================*/
/* Function: Read Measurement Window                                         */
/* Purpose:  returns the features and the status of the measurement window   */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Read_MeasurementWindow (ViSession instrID,ViPInt16 status,
                                                                   ViChar _VI_FAR message[],
                                                                   ViPReal64 Xposition,
                                                                   ViPReal64 Xwidth,
                                                                   ViPReal64 Yposition,
                                                                   ViPReal64 Ywidth)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    /*reads the status of the measurement window*/
    
    Fmt (Buffer,"%s<%s\r\n",s_readMeasWindow[4]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVR3271_status;
       
    if (Scan(Buffer,"%s>%d[b2]", status)!=1) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }
    
    Fmt(message,"%s<%s",s_textONOFF[*status]);
    
    /*reads the position of the window along the X-axis*/
    
    Fmt (Buffer,"%s<%s\r\n",s_readMeasWindow[0]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVR3271_status;
       
    if (Scan(Buffer,"%s>%f", Xposition)!=1) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }
     
    /*reads the width of the window along the X-axis*/ 
     
    Fmt (Buffer,"%s<%s\r\n",s_readMeasWindow[1]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVR3271_status;
       
    if (Scan(Buffer,"%s>%f", Xwidth)!=1) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }
    
    /*reads the position of the window along the Y-axis*/
    
    Fmt (Buffer,"%s<%s\r\n",s_readMeasWindow[2]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVR3271_status;
       
    if (Scan(Buffer,"%s>%f", Yposition)!=1) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }
     
    /*reads the width of the window along the X-axis*/ 
     
    Fmt (Buffer,"%s<%s\r\n",s_readMeasWindow[3]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVR3271_status;
       
    if (Scan(Buffer,"%s>%f", Ywidth)!=1) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }    
    
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Read Peak Options                                               */
/* Purpose:  reads the features of the peak search                           */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Read_Peak_Options (ViSession instrID, ViPInt16 contPeakStatus,
                                                               ViChar _VI_FAR contPeakMessage[],
                                                               ViPInt16 signalTrackStatus,
                                                               ViChar _VI_FAR signalTrackMessage[],
                                                               ViPReal64 deltaY)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    /*reads the status of the continuous peak search*/
    
    Fmt (Buffer,"%s<%s\r\n",s_readPeakOptions[0]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVR3271_status;
       
    if (Scan(Buffer,"%s>%d[b2]", contPeakStatus)!=1) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }
    
    Fmt(contPeakMessage,"%s<%s",s_textONOFF[*contPeakStatus]);
    
    /*reads the status of the signal track*/
    
    Fmt (Buffer,"%s<%s\r\n",s_readPeakOptions[1]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVR3271_status;
       
    if (Scan(Buffer,"%s>%d[b2]", signalTrackStatus)!=1) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }
    
    Fmt(signalTrackMessage,"%s<%s",s_textONOFF[*signalTrackStatus]);
    
    
    /*reads the value of deltaY*/
    
    Fmt (Buffer,"%s<%s\r\n",s_readPeakOptions[2]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVR3271_status;
        
    if (Scan(Buffer,"%s>%f", deltaY)!=1) {                                  
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }
     
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Read Trace Status                                               */
/* Purpose:  returns the status of the trace A or B                          */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Read_TraceStatus (ViSession instrID, ViBoolean parameter,
                                                              ViPInt16 status,
                                                              ViChar _VI_FAR message[])
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViUInt32 retCount;
    ViInt16 status1=0;
    ViByte Buffer[BUFFER_SIZE];
    
    if (AVR3271_invalidViBooleanRange (parameter))             
        return VI_ERROR_PARAMETER2; 
 
    Fmt (Buffer,"%s<%s\r\n",s_readTraceStatus[parameter]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVR3271_status;
       
    if (Scan(Buffer,"%s>%d[b2]", status)!=1) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }
    
    if ((*status/256)>=1)  status1 = (*status/256)+6;
    else status1 = *status;
     
    Fmt(message,"%s<%s",s_readTraceMessage[status1+parameter*3]); 
    
    return AVR3271_status; 
}

/*===========================================================================*/
/* Function: Read Detection Mode                                             */
/* Purpose:  returns the detector mode                                       */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Read_DetectionMode (ViSession instrID, ViPInt16 status,
                                                                ViChar _VI_FAR detectorMode[])
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_readDetectorMode[0]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVR3271_status;
       
    if (Scan(Buffer,"%s>%d[b2]", status)!=1) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }
     
    Fmt(detectorMode,"%s<%s",s_readDetectorMode[*status+1]); 
    
    return AVR3271_status; 
}

/*===========================================================================*/
/* Function: Limit Line Judgment                                             */           
/* Purpose:  returns if the test with the limit line(s) is OK                */
/*===========================================================================*/    
ViStatus _VI_FUNC AVR3271_Read_LLjudgment (ViSession instrID, ViPInt16 result,
                                                             ViChar _VI_FAR message[]) 
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_LLjudgement[0]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVR3271_status;
        
    if (Scan(Buffer,"%s>%d[b2]", result)!=1) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }
     
    Fmt(message,"%s<%s",s_judgementText[*result]); 
    
    return AVR3271_status; 
}

/*===========================================================================*/
/* Function: Read Limit Line Status                                          */
/* Purpose:  returns the status of the Limit Line                            */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Read_LimitLineStatus (ViSession instrID, ViBoolean parameter,
                        ViPInt16 status, ViChar _VI_FAR message[])
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViUInt32 retCount;
    ViByte Buffer[BUFFER_SIZE];
    
    if (AVR3271_invalidViBooleanRange (parameter))             
        return VI_ERROR_PARAMETER2; 
 
    Fmt (Buffer,"%s<%s\r\n",s_readLLstatus[parameter]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVR3271_status;
       
    if (Scan(Buffer,"%s>%d[b2]", status)!=1) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }
     
    Fmt(message,"%s<%s",s_textONOFF[*status]); 
    
    return AVR3271_status; 
}    

/*===========================================================================*/
/* Function: Read Level Correction Status                                    */
/* Purpose:  returns the level correction  status                            */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Read_LevelCorrectionStatus (ViSession instrID,ViPInt16 status,
                                                                       ViChar _VI_FAR message[])
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViUInt32 retCount;
    ViByte Buffer[BUFFER_SIZE];
    
    Fmt (Buffer,"%s<%s\r\n",s_readLevelCorrection[0]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVR3271_status;
       
    if (Scan(Buffer,"%s>%d[b2]", status)!=1) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }
    
    Fmt(message,"%s<%s",s_textONOFF[*status]); 
    
    return AVR3271_status;
} 
    
/*===========================================================================*/
/* Function: Read Marker Status                                              */
/* Purpose:  reads the marker status number -> text                          */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Read_MarkerStatus (ViSession instrID, ViPInt16 status,
                                                               ViChar _VI_FAR message[])
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_mkrNDsearch[0]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
       
    if ((AVR3271_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVR3271_status;
       
    if (Scan(Buffer,"%s>%d[b2]",status)!=1) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }   
    
    Fmt(message,"%s<%s", s_mkrNDstatus[*status]);  
    
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Read Delta Marker Options                                       */
/* Purpose:  returns the options status of the delta marker                  */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Read_DeltaMarkerOptions (ViSession instrID,
                                                  ViPInt16 fixedMarkerStatus,
                                                  ViChar _VI_FAR fixedMarkerMessage[],
                                                  ViPInt16 dividedMarkerStatus,
                                                  ViPReal64 dividedMarkerValue)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViUInt32 retCount=0;
    ViByte Buffer[BUFFER_SIZE];
    
    /*about the fixed delta marker*/
    
    Fmt (Buffer,"%s<%s\r\n",s_readMKDoptions[0]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVR3271_status;
       
    if (Scan(Buffer,"%s>%d[b2]", fixedMarkerStatus)!=1) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }
     
    Fmt(fixedMarkerMessage,"%s<%s",s_textONOFF[*fixedMarkerStatus]);
    
    /*about the 1/delta marker*/
    
    Fmt (Buffer,"%s<%s\r\n",s_readMKDoptions[1]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVR3271_status;
       
    if (Scan(Buffer,"%s>%d[b2x]%f",dividedMarkerStatus,dividedMarkerValue)!=1) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }  
    
    return AVR3271_status; 
}    
    

/*===========================================================================*/
/* Function: Read Marker Trace Status                                        */
/* Purpose:  returns the trace on which the markers are placed               */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Read_MarkerTraceStatus (ViSession instrID,ViPInt16 status,
                                                                   ViChar _VI_FAR message[])
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViUInt32 retCount;
    ViByte Buffer[BUFFER_SIZE];
    
    Fmt (Buffer,"%s<%s\r\n",s_readMarkerTraceStatus[0]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVR3271_status;
       
    if (Scan(Buffer,"%s>%d[b2]", status)!=1) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }
     
    Fmt(message,"%s<%s",s_readMarkerTraceStatus[*status+1]); 
    
    return AVR3271_status;
}
 
/*===========================================================================*/
/* Function: Read Multi Marker Status                                        */
/* Purpose:  reads the multi marker status & gives text message              */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Read_MultimarkersStatus (ViSession instrID,ViPInt16 status,
                                                                    ViChar _VI_FAR message[])
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_readmultiMkrStatus[0]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
       
    if ((AVR3271_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVR3271_status;
       
    if (Scan(Buffer,"%s>%d[b2]",status)!=1) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }   
    
    Fmt(message,"%s<%s", s_multiMkrMessage[*status]);  
    
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Read Trace Data                                                 */
/* Purpose:  returns trace A or B in an array                                */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Read_TraceData (ViSession instrID, ViBoolean trace,
                                                            ViBoolean format,
                                                            ViInt16 _VI_FAR traceArray[])
{
    ViStatus AVR3271_status = VI_SUCCESS;
    AVR3271_ptrFeatures featuresPtr;
    ViInt16 i;
    ViUInt32 retCount;
    ViByte dataBuffer[1405];
    ViByte wrtBuffer[BUFFER_SIZE];
    
    if (AVR3271_invalidViBooleanRange (trace))             
        return VI_ERROR_PARAMETER2;
        
    if (AVR3271_invalidViBooleanRange (format))             
        return VI_ERROR_PARAMETER3;
        
    if ((AVR3271_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3271_status;
       
    for (i=0;i<1405;i++) {
        dataBuffer[i]=0;
    } 
       
    if (format == VI_ON) { /*Binary Transfert*/
        Fmt (wrtBuffer,"%s<%s\r\n",s_traceBinary[trace]);     
        if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3271_status;
          
        if ((AVR3271_status = viRead (instrID,dataBuffer,(2*701)+2,&retCount)) < 0)
            return AVR3271_status;
       
        for (i=0;i<701;i++) {
            traceArray[i]=256*dataBuffer[2*i]+dataBuffer[2*i+1] ;
        }
    }
    
    else { /*ASCII Transfert*/ 
        Fmt (wrtBuffer,"%s<%s\r\n",s_traceASCII[trace]);     
        if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3271_status;
        for (i=0;i<701;i++) {
            if ((AVR3271_status = viRead (instrID, dataBuffer, 7, &retCount)) < 0)
                return AVR3271_status;
            if (Scan(dataBuffer,"%s>%d[b2]", &traceArray[i])!=1) { 
                AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
                return AVR3271_status;
            }   
        }
    }   
     
  return AVR3271_status; 
}

/*===========================================================================*/
/* Function: Set Trace Accuracy                                              */
/* Purpose:  chooses the value of the level's accuracy                       */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Set_TraceAccuracy (ViSession instrID, ViBoolean accuracy)
  
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVR3271_invalidViBooleanRange (accuracy))             
        return VI_ERROR_PARAMETER2;
    
    Fmt (Buffer,"%s<%s\r\n",s_traceAccuracy[accuracy]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
        
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Read Trace Accuracy                                             */
/* Purpose:  returns the value of the level's accuracy                       */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Read_TraceAccuracy (ViSession instrID, ViPInt16 accuracy)
  
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    ViInt16 status;
    
    Fmt (Buffer,"%s<%s\r\n",s_traceAccuracy[2]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
        
    if ((AVR3271_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVR3271_status;
       
    if (Scan(Buffer,"%s>%d[b2]", &status)!=1) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }
    
    if (status==0) *accuracy=341;
    else *accuracy=2721;
        
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: read normal marker                                              */
/* Purpose:  reads the normal marker position and level (without unit)       */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Read_NormalMarker (ViSession instrID,ViPReal64 MKRNposition,
                                             ViPReal64 MKRNlevel)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    /*reads the position of the marker normal*/
    
    Fmt (Buffer,"%s<%s\r\n",s_mkrNDsearch[1]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = viRead (instrID, Buffer, 20, &retCount)) < 0)
        return AVR3271_status;
       
    if (Scan(Buffer,"%s>%f", MKRNposition)!=1) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }
     
    /*reads the level of the marker normal*/ 
     
    Fmt (Buffer,"%s<%s\r\n",s_mkrNDsearch[2]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = viRead (instrID, Buffer, 20, &retCount)) < 0)
        return AVR3271_status;
       
    if (Scan(Buffer,"%s>%f", MKRNlevel)!=1) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }

    return AVR3271_status;
}

/*===========================================================================*/
/* Function: read delta marker                                               */
/* Purpose:  reads the position and level (without unit) of the delta marker */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Read_DeltaMarker (ViSession instrID,ViPReal64 MKRDposition,
                                            ViPReal64 MKRDlevel)
{

	ViStatus AVR3271_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    ViReal64 MKRMposition[8];
    ViReal64 MKRMlevel[8];
    
    /* search of the delta marker position */
    
    Fmt (Buffer,"%s<%s\r\n",s_multiMkrSearch[0]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;

    if ((AVR3271_status = viRead (instrID, Buffer, 200, &retCount)) < 0)
        return AVR3271_status;
    
    if (Scan(Buffer,"%s>%8f[x]%f",MKRMposition,MKRDposition)!=2) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }
     
    /* search of the  delta marker level*/
    
    Fmt (Buffer,"%s<%s\r\n",s_multiMkrSearch[1]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
       
    if ((AVR3271_status = viRead (instrID, Buffer, 200, &retCount)) < 0)
        return AVR3271_status;
       
    if (Scan(Buffer,"%s>%8f[x]%f",MKRMlevel,MKRDlevel)!=2) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }   

    return AVR3271_status;
}
 
/*===========================================================================*/
/* Function: Read Multi Markers                                              */
/* Purpose:  reads the positions and levels of the 6 multimarkers            */
/*           and the delta marker                                            */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Read_Multimarkers (ViSession instrID,ViReal64 _VI_FAR MKRMpositions[8],
                                                               ViPReal64 MKRDposition,
                                                               ViReal64 _VI_FAR MKRMlevels[8],
                                                               ViPReal64 MKRDlevel)
                                          
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    /* search of the multimarkers positions */
    
    Fmt (Buffer,"%s<%s\r\n",s_multiMkrSearch[0]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;

    if ((AVR3271_status = viRead (instrID, Buffer, 200, &retCount)) < 0)
        return AVR3271_status;
    
    if (Scan(Buffer,"%s>%8f[x]%f",MKRMpositions,MKRDposition)!=2) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }
     
    /* search of the multimarkers levels*/
    
    Fmt (Buffer,"%s<%s\r\n",s_multiMkrSearch[1]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
       
    if ((AVR3271_status = viRead (instrID, Buffer, 200, &retCount)) < 0)
        return AVR3271_status;
       
    if (Scan(Buffer,"%s>%8f[x]%f",MKRMlevels,MKRDlevel)!=2) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }   

    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Read MultiMarkerActive                                          */
/* Purpose:  searches the active Multi-Marker                                */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Read_MultiMarkerActive (ViSession instrID, ViPInt16 index)
                                      
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    ViInt16 i=0;
    ViReal64 MKRNposition;
    ViReal64 MKRMpositions[8];
    ViReal64 MKRDposition;
    ViReal64 MKRMlevels[8];
    ViReal64 MKRDlevel;
    
    
    Fmt (Buffer,"%s<%s\r\n",s_mkrNDsearch[1]);   
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3271_status;
    
    if ((AVR3271_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVR3271_status;
       
    if (Scan(Buffer,"%s>%f", &MKRNposition)!=1) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }
     
    AVR3271_Read_Multimarkers (instrID, MKRMpositions,&MKRDposition,
                                        MKRMlevels,&MKRDlevel);
    
    while ( (i<8)&&((MKRMpositions[i]!=MKRNposition)||(MKRMlevels[i]==0)) ) {
        i++;
    } 
    
    *index = i+1;
    
    return AVR3271_status; 
}

/*===========================================================================*/
/* Function: Write To Instrument                                             */
/* Purpose:  This function writes a command string to the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_WriteInstrData (ViSession instrID, ViString writeBuffer)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (wrtBuffer,"%s<%s\r\n",writeBuffer);     
    if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3271_status; 
    
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Read Instr Data                                                 */
/* Purpose:  This function reads the output buffer of the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_readInstrData (ViSession instrID, ViInt32 numBytes,
                                                           ViChar _VI_FAR readBuffer[],
                                                           ViPInt32 bytesRead)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    *bytesRead = 0L;
        
    if ((AVR3271_status = viRead (instrID, readBuffer, numBytes, bytesRead)) < 0)
        return AVR3271_status;

    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Reset                                                           */
/* Purpose:  This function resets the instrument.                            */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_reset (ViSession instrSession)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    AVR3271_ptrFeatures featuresPtr;
    
    if ((AVR3271_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &featuresPtr)) < 0)
        return AVR3271_status;
    
    /*  Initialize the instrument to a known state.  */
    Fmt (wrtBuffer,"%s<%s\r\n",s_reset[0]);     
    if ((AVR3271_status = viWrite(instrSession,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3271_status;
   
    if ((AVR3271_status = AVR3271_defaultInstrSetup (instrSession, featuresPtr->instrumentType)) < 0)  
        return AVR3271_status;
        
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Self-Test                                                       */
/* Purpose:  This function executes the instrument self-test and returns     */
/*           the result. The self test function is not supported by this     */
/*           instrument, and this function returns the warning               */
/*           VI_WARN_NSUP_SELF_TEST. (Self Test not supported)               */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_selfTest (ViSession instrID, ViPInt16 selfTestResult,
                                                      ViChar _VI_FAR selfTestMessage[])
{
    ViStatus AVR3271_status = VI_WARN_NSUP_SELF_TEST;
    
    Fmt (selfTestMessage, "%s<Self-Test Not Supported");
    *selfTestResult = -1;
    
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Error Query                                                     */
/* Purpose:  This function queries the instrument error queue, and returns   */
/*           the result.                                                     */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_errorQuery (ViSession instrID, ViPInt32 errorCode,
                                                        ViChar _VI_FAR errorMessage[])
{
    ViUInt32 retCnt = 0;
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_errorQuery[0]);     
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3271_status;
       
    if ((AVR3271_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVR3271_status;
       
    if (Scan(Buffer,"%s>%d[b4]", errorCode)!=1) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }
     
    Fmt (errorMessage,"%s<%d[b4]", *errorCode); 

    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Error Message                                                   */
/* Purpose:  This function translates the error return value from the        */
/*           instrument driver into a user-readable string.                  */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_errorMessage (ViSession instrID, ViStatus errorCode,
                                                          ViChar _VI_FAR errorMessage[])
 
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViInt16 i;
    
    static AVR3271_tStringValPair statusDescArray[] = 
    {
        {VI_WARN_NSUP_SELF_TEST,    "WARNING: Self-test not supported"},
        {VI_ERROR_PARAMETER1,        "ERROR: Parameter 1 out of range"},
        {VI_ERROR_PARAMETER2,        "ERROR: Parameter 2 out of range"},
        {VI_ERROR_PARAMETER3,        "ERROR: Parameter 3 out of range"},
        {VI_ERROR_PARAMETER4,        "ERROR: Parameter 4 out of range"},
        {VI_ERROR_PARAMETER5,        "ERROR: Parameter 5 out of range"},
        {VI_ERROR_PARAMETER6,        "ERROR: Parameter 6 out of range"},
        {VI_ERROR_PARAMETER7,        "ERROR: Parameter 7 out of range"},
        {VI_ERROR_PARAMETER8,        "ERROR: Parameter 8 out of range"},
        {VI_ERROR_INSTR_PARAMETER9 , "ERROR: Parameter 9 out of range"},
        {VI_ERROR_INSTR_PARAMETER10, "ERROR: Parameter 10 out of range"},
        {VI_ERROR_INSTR_PARAMETER11, "ERROR: Parameter 11 out of range"},
        {VI_ERROR_INSTR_PARAMETER12, "ERROR: Parameter 12 out of range"},
        {VI_ERROR_INSTR_PARAMETER13, "ERROR: Parameter 13 out of range"},
        {VI_ERROR_INSTR_PARAMETER14, "ERROR: Parameter 14 out of range"},
        {VI_ERROR_INSTR_PARAMETER15, "ERROR: Parameter 15 out of range"},
        {VI_ERROR_FAIL_ID_QUERY,    "ERROR: Identification query failed"},
        {VI_ERROR_INV_RESPONSE,     "ERROR: Interpreting instrument response"},
        {VI_ERROR_INSTR_FILE_OPEN,  "ERROR: Opening the specified file"},
        {VI_ERROR_INSTR_FILE_WRITE, "ERROR: Writing to the specified file"},
        {VI_ERROR_INSTR_INTERPRETING_RESPONSE, "ERROR: Interpreting the instrument's response"},
        {VI_ERROR_MARKERINACTIVE,   "ERROR: the chosen multi-marker is inactive"},
        {VI_NULL, VI_NULL}
     };

    AVR3271_status = viStatusDesc (instrID, errorCode, errorMessage);
    
    if (AVR3271_status == VI_WARN_UNKNOWN_STATUS) {
        for (i=0; statusDescArray[i].stringName; i++) {
            if (statusDescArray[i].stringVal == errorCode) {
                Fmt (errorMessage, "%s<%s", statusDescArray[i].stringName);
                return (VI_SUCCESS);
            }
        }
        Fmt (errorMessage, "%s<Unknown Error 0x%x[uw8p0]", errorCode); 
        return (VI_WARN_UNKNOWN_STATUS);
    }
    
    AVR3271_status = VI_SUCCESS;
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Revision Query                                                  */
/* Purpose:  This function returns the driver and instrument revisions.      */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_revisionQuery (ViSession instrID, ViChar _VI_FAR driverRev[],
                                                           ViChar _VI_FAR instrRev[])
{
    ViUInt32 retCount = 0;
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    
    Fmt (Buffer,"%s<%s\r\n",s_instrRevision[0]);     
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3271_status;
    
    if ((AVR3271_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVR3271_status;
       
    if (Scan(Buffer,"%s>%s", instrRev)!=1) { 
        AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3271_status;
    }
     
    Fmt (driverRev, "%s<%s", AVR3271_REVISION);
    
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Calibration                                                     */
/* Purpose:  Set calibration to element                                      */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Calibration (ViSession instrID, ViInt16 element,
                                                         ViReal64 level,
                                                         ViBoolean calCorrection,
                                                         ViBoolean freqCorrection)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVR3271_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    ViUInt32 timeout;
    
    
    if ((AVR3271_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3271_status;
    
    if (AVR3271_invalidViInt16Range (element,0,7))             
        return VI_ERROR_PARAMETER2;
        
    if (AVR3271_invalidViReal64Range (level,featuresPtr->calLevelMin,
                                           featuresPtr->calLevelMax))          
        return VI_ERROR_PARAMETER3;

    if (AVR3271_invalidViBooleanRange (calCorrection))             
        return VI_ERROR_PARAMETER4;
    
    if (AVR3271_invalidViBooleanRange (freqCorrection))             
        return VI_ERROR_PARAMETER5;
        
    /* stores the value of the timeout in variable timeout and then disables the timeout */
    if ((AVR3271_status = viGetAttribute (instrID,VI_ATTR_TMO_VALUE,&timeout)) < 0)
        return AVR3271_status;
    
    if ((AVR3271_status = viSetAttribute (instrID, VI_ATTR_TMO_VALUE, VI_TMO_INFINITE)) < 0)
            return AVR3271_status;
       
    /* element to calibrate*/  
    Fmt (wrtBuffer,"%s<%s%fDB,%s\r\n",s_calLevel[0],level,s_calibration[element]);     
    if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3271_status;     

    /* calibration and frequency correction */ 
    Fmt (wrtBuffer,"%s<%s,%s\r\n",s_freqCorrection[freqCorrection],
                                  s_calCorrection[calCorrection]);     
    if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3271_status;
    
    /* disables the calibration's signal */
    Fmt (wrtBuffer,"%s<%s\r\n",s_calLevel[1]);     
    if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3271_status;  
    
    /*  enables the timeout and sets its value to its last value */  
    if ((AVR3271_status = viSetAttribute (instrID, VI_ATTR_TMO_VALUE, timeout)) < 0)
        return AVR3271_status;
        
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Display label                                                   */
/* Purpose:  selects label                                                   */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Display_Label (ViSession instrID, ViBoolean showLabel,
                                                           ViString label)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVR3271_invalidViBooleanRange (showLabel))             
        return VI_ERROR_PARAMETER2;
       
    if (showLabel == VI_ON) {
        Fmt (wrtBuffer,"%s<%s,%s%s/\r\n",s_label[showLabel+1],s_label[showLabel+2],label);     
        if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3271_status; 
    }
    else {
        Fmt (wrtBuffer,"%s<%s,%s\r\n",s_label[showLabel],s_label[showLabel+1]);     
        if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3271_status;
    }
    
    return AVR3271_status;   
}

/*===========================================================================*/
/* Function : Read Status Byte                                               */
/* purpose : controls the status byte register                               */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Read_StatusByte (ViSession instrID, ViPInt16 statusByte)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViUInt16 status = 0;
    ViByte Buffer[BUFFER_SIZE];
    
    AVR3271_status = viReadSTB (instrID, &status);
    * statusByte = (ViInt16) status;
    
    return AVR3271_status;
}

/*===========================================================================*/
/* Function : Clear Status Byte                                              */
/* purpose : clears the status byte register                                 */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Clear_StatusByte (ViSession instrID)
{
 
    ViStatus AVR3271_status = VI_SUCCESS;
    ViUInt32 retCount = 0;
    ViByte Buffer[BUFFER_SIZE];
    
    Fmt (Buffer,"%s<%s\r\n",s_clearStatusByte[0]);     
    if ((AVR3271_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3271_status;
       
    return AVR3271_status;
    
}

/*===========================================================================*/
/* Function : Convert Frequency                                              */
/* Purpose : converts the frequency value returned in Hz by the analyzer     */
/*           in the value with the best unit (Hz, kHz, MHz or GHz)           */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Convert_Frequency (ViSession instrID,ViReal64 inputValue,
                                                              ViPReal64 outputValue,
                                                              ViChar _VI_FAR frequencyUnit[])
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViReal64 denom[]={1,1000,1000000,1000000000},interim[3];
    ViInt16 i;
    
    for (i=0;i<3;i++) {
        interim[i] = inputValue / denom[i+1];
    }
    
    i=0; 
    while ( (i<3)&&(fabs(interim[i])>=1) ) {
        i++;
    }
     
    *outputValue = inputValue / denom[i];
    
    Fmt(frequencyUnit,"%s<%s",s_freqUnit[i]); 
    
    return AVR3271_status;
} 
 
/*===========================================================================*/
/* Function : Convert Time                                                   */
/* Purpose : converts the time value returned in second(s) by the analyzer   */
/*           in the value with the best unit (ns, us, ms or s)               */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Convert_Time (ViSession instrID, ViReal64 inputValue,
                                                          ViPReal64 outputValue,
                                                          ViChar _VI_FAR timeUnit[])
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViReal64 compare[]={1000000000,1000000,1000,1};
    ViInt16 i;
    
    i=0;
    while ((i<=2)&&(fabs(inputValue)*compare[i]>=1000)) {
        i++;
    }
     
    *outputValue = inputValue*compare[i];
    Fmt(timeUnit,"%s<%s",s_timeUnit[i]);
    
    return AVR3271_status;
 
}  

/*===========================================================================*/
/* Function: Initialize card                                                 */
/* Purpose:  Initialize card on drive A or B                                 */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Init_MemoryCard (ViSession instrID, ViBoolean drive)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVR3271_invalidViBooleanRange (drive))          
        return VI_ERROR_PARAMETER2;
       
    Fmt (wrtBuffer,"%s<%s\r\n",s_initCard[drive]);     
    if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3271_status;      
    
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: select Memory Card                                              */
/* Purpose:  selects memory card drive                                       */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Select_MemoryCard (ViSession instrID, ViBoolean drive)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVR3271_invalidViBooleanRange (drive))
        return VI_ERROR_PARAMETER2;
       
    Fmt (wrtBuffer,"%s<%s\r\n",s_selectCard[drive]);     
    if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3271_status;    
    
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Copy Memory Card                                                */
/* Purpose:  Copy card A: to card B:                                         */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Copy_MemoryCard (ViSession instrID, ViBoolean parameter)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVR3271_invalidViBooleanRange (parameter))
        return VI_ERROR_PARAMETER2;
    
    Fmt (wrtBuffer,"%s<%s\r\n",s_copyCard[parameter]);     
    if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3271_status;
            
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: recall a file                                                   */
/* Purpose:  select the register or the file name to recall                  */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Recall_File (ViSession instrID, ViBoolean drive,
                                                         ViString fileName)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVR3271_invalidViBooleanRange (drive))             
        return VI_ERROR_PARAMETER2; 
    
    Fmt (wrtBuffer,"%s<%s%s/\r\n",s_recallFile[drive],fileName);     
    if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3271_status; 
    
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: save a file                                                     */
/* Purpose:  select the register or the file name to save                    */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_Save_File (ViSession instrID, ViBoolean drive,
                                                       ViString fileName)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVR3271_invalidViBooleanRange (drive))             
        return VI_ERROR_PARAMETER2; 
    
    Fmt (wrtBuffer,"%s<%s%s/\r\n",s_saveFile[drive],fileName);     
    if ((AVR3271_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3271_status; 
    
    return AVR3271_status;
    
}

/*===========================================================================*/
/* Function: Close                                                           */
/* Purpose:  This function closes the instrument.                            */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_close (ViSession instrSession)
{
    AVR3271_ptrFeatures featuresPtr;
    ViSession rmSession;
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (wrtBuffer,"%s<%s\r\n",s_close[0]);     
    if ((AVR3271_status = viWrite(instrSession,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3271_status;

    if ((AVR3271_status = viGetAttribute (instrSession,VI_ATTR_RM_SESSION,&rmSession))<0)
        return AVR3271_status;
    if ((AVR3271_status = viGetAttribute (instrSession,VI_ATTR_USER_DATA,&featuresPtr))<0)
        return AVR3271_status;
    
    if (featuresPtr != NULL) 
        free (featuresPtr);
    
    AVR3271_status = viClose (instrSession);
    viClose (rmSession);

    return AVR3271_status;
}

/*****************************************************************************/
/*= UTILITY ROUTINES (Non-Exportable Functions) =============================*/
/*****************************************************************************/


/*===========================================================================*/
/* Function: Boolean Value Out Of Range - ViBoolean                          */
/* Purpose:  This function checks a Boolean to see if it is equal to VI_TRUE */
/*           or VI_FALSE. If the value is out of range, the return value is  */
/*           VI_TRUE, otherwise the return value is VI_FALSE.                */
/*===========================================================================*/
ViBoolean AVR3271_invalidViBooleanRange (ViBoolean val)
{
    return ((val != VI_OFF && val != VI_ON) ? VI_ON : VI_OFF);
}

/*===========================================================================*/
/* Function: Short Signed Integer Value Out Of Range - ViInt16               */
/* Purpose:  This function checks a short signed integer value to see if it  */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean AVR3271_invalidViInt16Range (ViInt16 val, ViInt16 min, ViInt16 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Signed Integer Value Out Of Range - ViInt32                */
/* Purpose:  This function checks a long signed integer value to see if it   */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range,  the return value is VI_TRUE, otherwise the return    */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean AVR3271_invalidViInt32Range  (ViInt32 val, ViInt32 min, ViInt32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Short Unsigned Integer Value Out Of Range - ViUInt16            */
/* Purpose:  This function checks a short unsigned integer value to see if it*/  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range,  the return value is VI_TRUE, otherwise the return    */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean AVR3271_invalidViUInt16Range  (ViUInt16 val, ViUInt16 min, ViUInt16 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Unsigned Integer Value Out Of Range - ViUInt32             */
/* Purpose:  This function checks a long unsigned integer value to see if it */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range,  the return value is VI_TRUE, otherwise the return    */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean AVR3271_invalidViUInt32Range  (ViUInt32 val, ViUInt32 min, ViUInt32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}
/*===========================================================================*/
/* Function: Real (Float) Value Out Of Range - ViReal32                      */
/* Purpose:  This function checks a real (float) value to see if it lies     */  
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean AVR3271_invalidViReal32Range  (ViReal32 val, ViReal32 min, ViReal32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (Double) Value Out Of Range - ViReal64                     */
/* Purpose:  This function checks a real (double) value to see if it lies    */  
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean AVR3271_invalidViReal64Range  (ViReal64 val, ViReal64 min, ViReal64 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Initialize Clean Up                                             */
/* Purpose:  This function is used only by the AVR3271_init function.  When   */
/*           an error is detected this function is called to close the       */
/*           open resource manager and instrument object sessions and to     */
/*           set the instrSession that is returned from AVR3271_init to       */
/*           VI_NULL.                                                        */
/*===========================================================================*/
ViStatus AVR3271_initCleanUp (ViSession openRMSession,
                             ViPSession openInstrSession, ViStatus currentStatus)
{
    viClose (*openInstrSession);
    viClose (openRMSession);
    *openInstrSession = VI_NULL;
    
    return currentStatus;
    
}


/*===========================================================================*/
/* function  Search Scale                                                    */
/* Purpose : search the scale used by the analyzer when the user begin       */
/*           the remote control                                              */
/*===========================================================================*/
ViStatus AVR3271_Search_Scale (ViSession instrSession, ViPInt16 scale)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViUInt32 retCount;
    ViByte Buffer[BUFFER_SIZE];
    ViReal64  RBW[10] = {100, 300, 1000, 3000, 10000, 30000, 100000, 300000, 1000000, 3000000};
    ViReal64  RBWvaluestart, RBWvaluestop;
    ViInt16   i, RBWcontrolstart, RBWcontrolstop;
    
    /* returns the value and the status (auto/manual) of RBW */
    AVR3271_Read_RBWandVBW (instrSession, 1, &RBWvaluestart, &RBWcontrolstart);
    /* send RBW = 1 kHz to the analyzer */
    AVR3271_Set_RBW (instrSession, 3);
    /* returns the value of RBW : 1 kHz if log scale and 3 kHz if linear scale */
    AVR3271_Read_RBWandVBW (instrSession, 1, &RBWvaluestop, &RBWcontrolstop);
    
    if (RBWvaluestop != 1000) {   /* we are in linear scale */
        *scale = 4; 
    }
    else {  /* we are in logarithmic scale and we are looking for the log scale used */
        Fmt (Buffer,"%s<%s\r\n",s_yScale[5]);   
        if ((AVR3271_status = viWrite(instrSession,Buffer,NumFmtdBytes(),&retCount))<0)
            return AVR3271_status;
        if ((AVR3271_status = viRead (instrSession, Buffer, 25, &retCount)) < 0)
            return AVR3271_status;
        if (Scan(Buffer,"%s>%d[b2]", scale)!=1) { 
            AVR3271_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
            return AVR3271_status;
        }
    }   
    
    /* send the previous value of RBW */
    if (RBWcontrolstart == 1) {
        AVR3271_Set_RBW (instrSession, 0);
    }
    else {
        i = 0;
        while (RBW[i] != RBWvaluestart) {
            i++;
        }
        AVR3271_Set_RBW (instrSession, i+1);
    }
    
    return AVR3271_status;
  
} 

/*****************************************************************************/
/*----------- INSERT INSTRUMENT-DEPENDENT UTILITY ROUTINES HERE -------------*/
/*****************************************************************************/


/*===========================================================================*/
/* Function: Internal Reset                                                  */
/* Purpose:  This function resets the instrument.                            */
/*===========================================================================*/
ViStatus AVR3271_initReset (ViSession instrSession,ViUInt32 instrType)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    /*  Initialize the instrument to a known state.  */
    Fmt (wrtBuffer,"%s<%s\r\n",s_reset[0]);     
    if ((AVR3271_status = viWrite(instrSession,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3271_status;
   
    if ((AVR3271_status = AVR3271_defaultInstrSetup (instrSession,instrType)) < 0)  
        return AVR3271_status;
        
    return AVR3271_status;
}

/*===========================================================================*/
/* Function: Default Instrument Setup                                        */
/* Purpose:  This function sends a default setup to the instrument.  This    */
/*           function is called by the AVR3271_reset operation and by the     */
/*           AVR3271_init function if the reset option has not been           */
/*           selected.  This function is useful for configuring any          */
/*           instrument settings that are required by the rest of the        */
/*           instrument driver functions such as turning headers ON or OFF   */
/*           or using the long or short form for commands, queries, and data.*/                                    
/*===========================================================================*/
ViStatus AVR3271_defaultInstrSetup (ViSession instrSession, ViUInt32 instrType)
{
    ViStatus AVR3271_status = VI_SUCCESS;
    ViUInt32 retCount = 0;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVR3271_ptrFeatures featuresPtr;
    ViInt16 i;
    ViReal64 MKRMpositions[8],positionDelta,MKRMlevels[8];
    ViReal64 levelDelta, span;
    ViChar unitName[10];
    
    /* Determine if the structure has been initialized for the current VISA  */
    /* Session and malloc if it has not.                                     */
    if (AVR3271_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &featuresPtr))
        return AVR3271_status;
    
    if (featuresPtr == NULL) 
        featuresPtr = malloc (sizeof (struct AVR3271_instrFeatures));
    
    /*delete the soft menus*/
    Fmt (wrtBuffer,"%s<%s\r\n",s_reset[1]);     
    if ((AVR3271_status = viWrite(instrSession,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3271_status;
        
        /* the marker display mode is set to absolute*/
    Fmt (wrtBuffer,"%s<%s",s_displayMode[0]);     
    if ((AVR3271_status = viWrite(instrSession,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3271_status;
       
    /* the minimum attenuation is controled by the analyzer*/
    Fmt (wrtBuffer,"%s<%s",s_attMin[1]);     
    if ((AVR3271_status = viWrite(instrSession,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3271_status;  
        
    switch (instrType) {
    
        case 3271 :
            featuresPtr->frequencyLow[0] = 0;
            featuresPtr->frequencyHigh[0] = 26.5e9;
            featuresPtr->frequencyLow[1]= -500e6;
            featuresPtr->frequencyHigh[1] = 26.5e9;
            featuresPtr->frequencyLow[2] = 100;
            featuresPtr->frequencyHigh[2] = 31.8e9;
            featuresPtr->spanLow = 200;
            featuresPtr->spanHigh = 27.5e9;
            featuresPtr->sweepTimeMini[0] = 50e-6;
            featuresPtr->sweepTimeMini[1] = 20e-3;
            featuresPtr->sweepTimeMaxi[0] = 1000;
            featuresPtr->sweepTimeMaxi[1] = 1000;
            /* set the scale to 10 dB */
            featuresPtr->scale=0;
            /* min att controlled by the analyzer*/
            featuresPtr->minAttControl = 1;
            break;
            
        case 3265 :
           	featuresPtr->frequencyLow[0] = 0;
            featuresPtr->frequencyHigh[0] = 8e9;
            featuresPtr->frequencyLow[1]= -500e6;
            featuresPtr->frequencyHigh[1] = 8e9;
            featuresPtr->frequencyLow[2] = 100;
            featuresPtr->frequencyHigh[2] = 8e9;
            featuresPtr->spanLow = 200;
            featuresPtr->spanHigh = 8e9;
            featuresPtr->sweepTimeMini[0] = 50e-6;
            featuresPtr->sweepTimeMini[1] = 20e-3;
            featuresPtr->sweepTimeMaxi[0] = 1000;
            featuresPtr->sweepTimeMaxi[1] = 1000; 
    }
    
    featuresPtr->instrumentType = instrType;
    featuresPtr->freqOffsetMin = -100e6;   /*Hz*/
    featuresPtr->freqOffsetMax = +100e6;
    featuresPtr->refLevelOffsetMin = -100; /*dB*/
    featuresPtr->refLevelOffsetMax = +100;
    featuresPtr->inputAttMin = 0;   /*dB*/
    featuresPtr->inputAttMax = 70;
    featuresPtr->gatePositionMini = 1e-6;
    featuresPtr->gatePositionMaxi = 65e-3;
    featuresPtr->gateWidthMini = 2e-6;
    featuresPtr->gateWidthMaxi = 65e-3;
    featuresPtr->averageCountMini = 2;
    featuresPtr->averageCountMaxi = 1000;
    featuresPtr->DL_LevelMin = -1000;
    featuresPtr->DL_LevelMax = 1000;
    featuresPtr->refLevelMin[0]=-149.9;
    featuresPtr->refLevelMin[1]=-102.9;
    featuresPtr->refLevelMin[2]=-42.9;
    featuresPtr->refLevelMin[3]=-36.9;
    featuresPtr->refLevelMin[4]=-59.9;
    featuresPtr->refLevelMin[5]=0;
    featuresPtr->refLevelMin[6]=7.153e-9;
    featuresPtr->refLevelMin[7]=1.023e-18;
    featuresPtr->refLevelMax[0]=60;
    featuresPtr->refLevelMax[1]=107;
    featuresPtr->refLevelMax[2]=167;
    featuresPtr->refLevelMax[3]=173;
    featuresPtr->refLevelMax[4]=150;
    featuresPtr->refLevelMax[5]=0;
    featuresPtr->refLevelMax[6]=223.6;
    featuresPtr->refLevelMax[7]=1000;
    featuresPtr->displayMode=0; /*displayMode->absolute*/
    featuresPtr->deltaYmin=0.1;
    featuresPtr->deltaYmax=10;
    featuresPtr->calLevelMin=-30;
    featuresPtr->calLevelMax=-10;
    
    AVR3271_Read_RefUnit (instrSession, &featuresPtr->refUnit,unitName);
    
    AVR3271_Read_FreqSettings (instrSession, 1, &span);
    if (span==0) featuresPtr->span=0;
    else featuresPtr->span=1;
    
    AVR3271_Read_Multimarkers (instrSession,MKRMpositions,&positionDelta,
                                            MKRMlevels,&levelDelta);
                                         
    for (i=0;i<8;i++) {
        if ((MKRMpositions[i]!=0)||(MKRMlevels[i]!=0)) featuresPtr->MKRMstatus[i]=1;
        else featuresPtr->MKRMstatus[i]=0;
    }
    
    
    Fmt (featuresPtr -> instrDriverRevision, "%s<%s", AVR3271_REVISION);    
    
    if (viSetAttribute (instrSession, VI_ATTR_USER_DATA, (ViUInt32)featuresPtr)!= 0)
        return AVR3271_status;
       
    return AVR3271_status;
    
}

/*****************************************************************************/
/*=== END INSTRUMENT DRIVER SOURCE CODE =====================================*/
/*****************************************************************************/
