/*== ADVANTEST ULIS Series Spectrum Analyzers Include File ==================*/

/*===========================================================================*/
/*  Please do not use global variables or arrays in the include file         */
/*  instrument drivers.                                                      */
/*===========================================================================*/
     
#ifndef __AVR3271_HEADER
#define __AVR3271_HEADER

#include <vpptype.h>

#if defined(__cplusplus) || defined(__cplusplus__)
extern "C" {
#endif

/*===========================================================================*/
/* Instrument Specific Error Codes                                           */
/*===========================================================================*/
#define VI_ERROR_INSTR_FILE_OPEN              (_VI_ERROR+0x3FFC0800L)
#define VI_ERROR_INSTR_FILE_WRITE             (_VI_ERROR+0x3FFC0801L)

#define VI_ERROR_INSTR_INTERPRETING_RESPONSE  (_VI_ERROR+0x3FFC0803L)
#define VI_ERROR_INSTR_PARAMETER9             (_VI_ERROR+0x3FFC0809L)
#define VI_ERROR_INSTR_PARAMETER10            (_VI_ERROR+0x3FFC080AL)
#define VI_ERROR_INSTR_PARAMETER11            (_VI_ERROR+0x3FFC080BL)
#define VI_ERROR_INSTR_PARAMETER12            (_VI_ERROR+0x3FFC080CL)
#define VI_ERROR_INSTR_PARAMETER13            (_VI_ERROR+0x3FFC080DL)
#define VI_ERROR_INSTR_PARAMETER14            (_VI_ERROR+0x3FFC080EL)
#define VI_ERROR_INSTR_PARAMETER15            (_VI_ERROR+0x3FFC080FL)

#define VI_WARNING_INSTR_OFFSET                         (0x3FFC0900L) 
#define VI_ERROR_INSTR_OFFSET                 (_VI_ERROR+0x3FFC0900L)

#define PREFIX_ERROR_INVALID_CONFIGURATION    (VI_ERROR_INSTR_OFFSET + 0xF0L)

#define VI_ERROR_MARKERINACTIVE               (_VI_ERROR+0x3FFC0810L)

/*===========================================================================*/
/* CONSTANTS FOR instrFeatures STRUCTURE                                     */
/*===========================================================================*/
#define FREQUENCYLOW 1
#define FREQUENCYHIGH 2
#define SPANLOW 3
#define SPANHIGH 4
#define FREQOFFSETMIN 5
#define FREQOFFSETMAX 6
#define INPUTATTMIN 7
#define INPUTATTMAX 8
#define REFLEVELMIN 9
#define REFLEVELMAX 10
#define REFLEVELOFFSETMIN 11
#define REFLEVELOFFSETMAX 12
#define SWEEPTIMEMINI 13
#define SWEEPTIMEMAXI 14
#define GATEPOSITIONMINI 15
#define GATEPOSITIONMAXI 16
#define GATEWIDTHMINI 17
#define GATEWIDTHMAXI 18
#define AVERAGECOUNTMINI 19
#define AVERAGECOUNTMAXI 20
#define DLLEVELMIN 21
#define DLLEVELMAX 22
#define DISPLAYMODE 23
#define REFUNIT 24
#define SPAN 25
#define SCALE 26
#define DELTAYMIN 27
#define DELTAYMAX 28
#define CALLEVELMIN 29
#define CALLEVELMAX 30
#define MKRMSTATUS 31


/*===========================================================================*/
/* GLOBAL USER-CALLABLE FUNCTION DECLARATIONS (Exportable Functions)         */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3271_init (ViRsrc resourceName,
                               ViBoolean IDquery,   
                               ViBoolean resetDevice,
                               ViPSession instrID);
                             
ViStatus _VI_FUNC AVR3271_configRead(ViSession instrID,
                                    ViBoolean trace,
                                    ViInt16 peakSearch,
                                    ViPInt16 traceAccuracy,
                                    ViInt16 _VI_FAR traceArray[],
                                    ViPReal64 sweepTime,
                                    ViChar _VI_FAR sweepUnit[],
                                    ViPReal64 MKRNposition,
                                    ViPReal64 MKRNlevel);

/*==============================*/
/*   Configurations Fonctions   */
/*==============================*/

/* Configure Frequency */                    
                    
ViStatus _VI_FUNC AVR3271_Set_Freq (ViSession instrID,
                                   ViInt16 frequencyType,
                                   ViReal64 frequencyValue);
                                 
ViStatus _VI_FUNC AVR3271_Set_Span (ViSession instrID,
                                   ViInt16 spanType,
                                   ViReal64 spanValue);
                                 
ViStatus _VI_FUNC AVR3271_Set_FreqOffset (ViSession instrID,
                                         ViBoolean activation,
                                         ViReal64 frequencyOffset);
                                       
ViStatus _VI_FUNC AVR3271_Set_FreqStepSize (ViSession instrID,
                                           ViBoolean controlType,
                                           ViReal64 frequencyStep);
                                         
ViStatus _VI_FUNC AVR3271_Set_FreqCounter (ViSession instrID,
                                          ViBoolean activation,
                                          ViInt16 resolution);
                               
/* Configure Amplitude */

ViStatus _VI_FUNC AVR3271_Set_RefLevel (ViSession instrID,
                                       ViReal64 refLevelValue);

ViStatus _VI_FUNC AVR3271_Set_RefUnit (ViSession instrID,
                                      ViInt16 referenceUnit);

ViStatus _VI_FUNC AVR3271_Set_Scale (ViSession instrID,
                                    ViInt16 scale);

ViStatus _VI_FUNC AVR3271_Set_RefOffset (ViSession instrID,
                                        ViBoolean activation,
                                        ViReal64 offset);
                                      
ViStatus _VI_FUNC AVR3271_Set_Att (ViSession instrID,
                                  ViBoolean controlType,
                                  ViReal64 value);
                                  
ViStatus _VI_FUNC AVR3271_Set_MinATT (ViSession instrID,
									  ViBoolean controlType,
                                      ViReal64 minInputAttValue);                                  
                                  
ViStatus _VI_FUNC AVR3271_Set_HighSenceMode (ViSession instrID,
											ViBoolean mode);                                   

/* Configure Bandwidth */

ViStatus _VI_FUNC AVR3271_Set_RBW (ViSession instrID,
                                  ViInt16 RBWSetting);

ViStatus _VI_FUNC AVR3271_Set_VBW (ViSession instrID,
                                  ViInt16 VBWSetting);

ViStatus _VI_FUNC AVR3271_Set_All_Auto_Couple (ViSession instrID);

/* Configure Sweep */

ViStatus _VI_FUNC AVR3271_Set_SweepTime (ViSession instrID,
                                        ViBoolean controlType,
                                        ViReal64 sweepTime);
                                       
ViStatus _VI_FUNC AVR3271_Set_SweepMode (ViSession instrID,
                                        ViBoolean sweepMode,
                                        ViBoolean windowSweep);
                                        
ViStatus _VI_FUNC AVR3271_Set_Trigger(ViSession instrID,
                                      ViInt16 source,
                                      ViBoolean polarity,
                                      ViInt32 videoLevel);
                                     
ViStatus _VI_FUNC AVR3271_Set_TriggerPosition(ViSession instrID,
                                             ViInt16 position);                                     
                                       
ViStatus _VI_FUNC AVR3271_Set_TriggerTV(ViSession instrID,
                                       ViInt16 triggerSource,
                                       ViBoolean triggerPolarity,
                                       ViInt16 lineNumber);
                                       
/* Configure Trace */

ViStatus _VI_FUNC AVR3271_Set_TraceAMode (ViSession instrID,
                                          ViInt16 traceAMode,
                                          ViInt16 averageCount,
                                          ViBoolean averageMode);
                                       
ViStatus _VI_FUNC AVR3271_Set_TraceBMode (ViSession instrID,
										  ViInt16 traceBMode,
                                          ViInt16 averageCount,
                                          ViBoolean averageMode);

ViStatus _VI_FUNC AVR3271_Set_TraceDetector (ViSession instrID,
                                             ViInt16 detectorMode);

/* Configure Display & Limit Lines */

ViStatus _VI_FUNC AVR3271_Set_LimitLine (ViSession instrID,
                                        ViBoolean limitLine,
                                        ViString LL_FreqTable,
                                        ViString LL_LevelTable);
                                     
ViStatus _VI_FUNC AVR3271_Activate_LimitLine (ViSession instrID,
                                             ViBoolean LL1Status,
                                             ViBoolean LL2Status,
                                             ViBoolean levelXaxis,
                                             ViBoolean levelYaxis,
                                             ViBoolean passfail);
                                                             

ViStatus _VI_FUNC AVR3271_Set_DisplayLine (ViSession instrID,
                                          ViBoolean showDL,
                                          ViReal64 DLposition);
                               
/* Configure Measurement Window */

ViStatus _VI_FUNC AVR3271_Set_MeasurementWindow (ViSession instrID,
                                                ViBoolean windowStatus,
                                                ViReal64 Xposition,
                                                ViReal64 Xwidth,
                                                ViReal64 Yposition,
                                                ViReal64 Ywidth);
                                     
/* Configure Level Correction */

ViStatus _VI_FUNC AVR3271_Set_LevelCorrection (ViSession instrID,
                                              ViBoolean LCstatus,
                                              ViString LC_FreqTable,
                                              ViString LC_LevelTable);

/* Configure Markers Normal & Delta */

ViStatus _VI_FUNC AVR3271_Set_NormalMarker (ViSession instrID,
                                           ViReal64 position);
                                         
ViStatus _VI_FUNC AVR3271_Set_DeltaMarker (ViSession instrID,
                                          ViReal64 position);
                                                           
ViStatus _VI_FUNC AVR3271_Set_DeltaMarkerOptions (ViSession instrID,
                                                 ViBoolean divided,
                                                 ViBoolean fixed);
                                               
ViStatus _VI_FUNC AVR3271_Set_MarkerTrace (ViSession instrID,
                                          ViBoolean markerTrace);

/* Configure Multimarkers */

ViStatus _VI_FUNC AVR3271_Activate_Multimarkers (ViSession instrID);

ViStatus _VI_FUNC AVR3271_Set_Multimarkers (ViSession instrID,
                                           ViString MKRMstatus,
                                           ViString MKRMpositions);
                                                            
ViStatus _VI_FUNC AVR3271_Set_ActiveMultiMarker (ViSession instrID,
                                                ViInt16 MKRMactive);
                                                
ViStatus _VI_FUNC AVR3271_Delete_Markers (ViSession instrID);                                                
                                     
/* Configure Peak Search */

ViStatus _VI_FUNC AVR3271_Set_PeakSearchOptions (ViSession instrID,
                                                ViBoolean continuousSearch, 
                                                ViBoolean signalTrack,
                                                ViInt16 peakRange,
                                                ViReal64 deltaY);
                                     
/*=============================*/
/*   Action/Status Functions   */
/*=============================*/

ViStatus _VI_FUNC AVR3271_Search_Peak (ViSession instrID,
                                      ViInt16 searchType);
                                    
ViStatus _VI_FUNC AVR3271_Execute_PeakTo (ViSession instrID,
                                         ViBoolean peakTo);
                                       
ViStatus _VI_FUNC AVR3271_Set_MarkerDeltaTo (ViSession instrID,
                                            ViInt16 markerDeltaTo);

/* Trace Action */

ViStatus _VI_FUNC AVR3271_Take_Sweep (ViSession instrID,
                                     ViInt16 startSweep);
                                   
ViStatus _VI_FUNC AVR3271_Execute_TraceMath (ViSession instrID,
                                            ViInt16 traceMath);
                                          
ViStatus _VI_FUNC AVR3271_NormalizingMode (ViSession instrID,
                                          ViInt16 mode);     
                                          
                                          
ViStatus _VI_FUNC AVR3271_Control_TraceAverage (ViSession instrID,
                                               ViInt16 traceAverage);
                                    
/* Configuration Read */

ViStatus _VI_FUNC AVR3271_Read_FreqSettings (ViSession instrID,
                                            ViInt16 frequencyParameter,
                                            ViPReal64 frequencyValue);
                                          
ViStatus _VI_FUNC AVR3271_Read_FreqOffset (ViSession instrID,
                                          ViPInt16 status,
                                          ViPReal64 value);
                                          
ViStatus _VI_FUNC AVR3271_Read_FreqStepSize (ViSession instrID,
                                            ViPReal64 frequencyStepSize,
                                            ViPInt16 controlStatus);
                                          
ViStatus _VI_FUNC AVR3271_Read_FreqCounter (ViSession instrID,
                                           ViPReal64 counterValue,
                                           ViPInt16 counterStatus);
                                         
ViStatus _VI_FUNC AVR3271_Read_ReferenceLevel(ViSession instrID,
                                             ViPReal64 refLevelValue);

ViStatus _VI_FUNC AVR3271_Read_RefLevelOffset (ViSession instrID,
                                              ViPReal64 value,
                                              ViPInt16 controlType);
                                            
ViStatus _VI_FUNC AVR3271_Read_RefUnit (ViSession instrID,
                                       ViPInt16 referenceUnitNumber,
                                       ViChar _VI_FAR referenceUnitName[]);
                                     
ViStatus _VI_FUNC AVR3271_Read_Scale (ViSession instrID,
                                     ViPInt16 status,
                                     ViChar _VI_FAR scaleName[]);
                                   
ViStatus _VI_FUNC AVR3271_Read_Attenuation (ViSession instrID,
                                           ViPReal64 valuedB,
                                           ViPInt16 controlType);
                                         
ViStatus _VI_FUNC AVR3271_Read_RBWandVBW (ViSession instrID,
                                         ViBoolean bandwidthtoRead,
                                         ViPReal64 bandwidthValue,
                                         ViPInt16 controlType);
                                       
ViStatus _VI_FUNC AVR3271_Read_SweepTime (ViSession instrID,
                                         ViPReal64 timeValue,
                                         ViPInt16 controlType);
                                       
ViStatus _VI_FUNC AVR3271_Read_SweepMode (ViSession instrID,
                                         ViPInt16 sweepModeStatus,
                                         ViChar _VI_FAR sweepMode[]);
                                       
ViStatus _VI_FUNC AVR3271_Read_Trigger (ViSession instrID,
                                       ViPInt16 triggerStatus,
                                       ViChar _VI_FAR triggerName[],
                                       ViPReal64 triggerFeature);
                                       
ViStatus _VI_FUNC AVR3271_Read_TriggerPosition (ViSession instrID, ViPInt16 triggerPosition);                                       
                                         
ViStatus _VI_FUNC AVR3271_Read_DisplayLine(ViSession instrID,
                                          ViPReal64 level,
                                          ViPInt16 status,
                                          ViChar _VI_FAR message[]);
                                        
ViStatus _VI_FUNC AVR3271_Read_LimitLineOptions (ViSession instrID,
                                                ViBoolean parameter,
                                                ViPInt16 levelStatus,
                                                ViChar _VI_FAR levelMessage[]);
                                              
ViStatus _VI_FUNC AVR3271_Read_MeasurementWindow (ViSession instrID,
                                                 ViPInt16 status,
                                                 ViChar _VI_FAR message[],
                                                 ViPReal64 Xposition,
                                                 ViPReal64 Xwidth,
                                                 ViPReal64 Yposition,
                                                 ViPReal64 Ywidth);
                                               
ViStatus _VI_FUNC AVR3271_Read_Peak_Options (ViSession instrID,
                                            ViPInt16 contPeakStatus,
                                            ViChar _VI_FAR contPeakMessage[],
                                            ViPInt16 signalTrackStatus,
                                            ViChar _VI_FAR signalTrackMessage[],
                                            ViPReal64 deltaY);

/* Status Read */

ViStatus _VI_FUNC AVR3271_Read_TraceStatus (ViSession instrID,
                                           ViBoolean traceModetoRead,
                                           ViPInt16 traceModeStatus,
                                           ViChar generalStatusText[]);
                                         
ViStatus _VI_FUNC AVR3271_Read_DetectionMode (ViSession instrID,
                                             ViPInt16 detectorModeStatus,
                                             ViChar _VI_FAR detectorMode[]);
                                             
ViStatus _VI_FUNC AVR3271_Read_LLjudgment (ViSession instrID, ViPInt16 result,
                                                             ViChar _VI_FAR message[]);
                                                             
ViStatus _VI_FUNC AVR3271_Read_LimitLineStatus (ViSession instrID,
                                               ViBoolean limitLinetoCheck,
                                               ViPInt16 limitLineStatus,
                                               ViChar _VI_FAR limitLineMessage[]); 
                                             
ViStatus _VI_FUNC AVR3271_Read_LevelCorrectionStatus (ViSession instrID,
                                                     ViPInt16 LCstatus,
                                                     ViChar _VI_FAR LCmessage[]);
                                                   
ViStatus _VI_FUNC AVR3271_Read_MarkerStatus (ViSession instrID,
                                            ViPInt16 markerStatus,
                                            ViChar _VI_FAR markerStatusMessage[]);
ViStatus _VI_FUNC AVR3271_Read_DeltaMarkerOptions (ViSession instrID,
                                                  ViPInt16 fixedMarkerStatus,
                                                  ViChar _VI_FAR fixedMarkerMessage[],
                                                  ViPInt16 dividedMerkerStatus,
                                                  ViPReal64 dividedMarkerValue);                                            

ViStatus _VI_FUNC AVR3271_Read_MarkerTraceStatus (ViSession instrID,
                                                 ViPInt16 markerTraceStatus,
                                                 ViChar _VI_FAR markerTraceMessage[]);
                                               
ViStatus _VI_FUNC AVR3271_Read_MultimarkersStatus (ViSession instrID,
                                                  ViPInt16 multimarkersStatus,
                                                  ViChar _VI_FAR multimarkersMessage[]);
                                       
/*====================*/
/*   Data Functions   */
/*====================*/

/* Trace Function */

ViStatus _VI_FUNC AVR3271_Read_TraceData (ViSession instrID,
                                          ViBoolean tracetoRead,
                                          ViBoolean dataFormat,
                                          ViInt16 _VI_FAR traceDataArray[]);
                                       
ViStatus _VI_FUNC AVR3271_Set_TraceAccuracy (ViSession instrID,
										     ViBoolean accuracy);                                        
                                       
ViStatus _VI_FUNC AVR3271_Read_TraceAccuracy (ViSession instrID,
											  ViPInt16 accuracy);                                        
                              
/* Markers Functions */

ViStatus _VI_FUNC AVR3271_Read_NormalMarker (ViSession instrID,
                                             ViPReal64 position,
                                             ViPReal64 level);
                                          
ViStatus _VI_FUNC AVR3271_Read_DeltaMarker (ViSession instrID,
                                            ViPReal64 position,
                                            ViPReal64 level);
                                         
ViStatus _VI_FUNC AVR3271_Read_Multimarkers (ViSession instrID,
                                             ViReal64 _VI_FAR MKRMpositions[],
                                             ViPReal64 MKRDposition,
                                             ViReal64 _VI_FAR MKRMlevels[],
                                             ViPReal64 MKRDlevel);
                                          
ViStatus _VI_FUNC AVR3271_Read_MultiMarkerActive (ViSession instrID,
                                                  ViPInt16 multiMarkerActive);
                                      
/*=======================*/               
/*   Utility Functions   */
/*=======================*/

ViStatus _VI_FUNC AVR3271_Calibration (ViSession instrID,
                                      ViInt16 elementtoCalibrate,
                                      ViReal64 calibrationLevel,
                                      ViBoolean calibrationCorrection,
                                      ViBoolean frequencyCorrection);
                                    
ViStatus _VI_FUNC AVR3271_Display_Label (ViSession instrID,
                                        ViBoolean showLabel,
                                        ViString label);

ViStatus _VI_FUNC AVR3271_Read_StatusByte (ViSession instrID,
                                          ViPInt16 statusByteRead);
                                          
ViStatus _VI_FUNC AVR3271_Clear_StatusByte (ViSession instrID);                                          

/* Frequency or Time Conversion */

ViStatus _VI_FUNC AVR3271_Convert_Frequency (ViSession instrID,
                                            ViReal64 inputFrequencyValue,
                                            ViPReal64 outputFrequencyValue,
                                            ViChar _VI_FAR frequencyUnit[]);
                                          
ViStatus _VI_FUNC AVR3271_Convert_Time (ViSession instrID,
                                       ViReal64 inputTimeValue,
                                       ViPReal64 outputTimeValue,
                                       ViChar _VI_FAR timeUnit[]);
                            
/* Memory Card Management */

ViStatus _VI_FUNC AVR3271_Init_MemoryCard (ViSession instrID,
                                          ViBoolean drive);

ViStatus _VI_FUNC AVR3271_Select_MemoryCard (ViSession instrID,
                                            ViBoolean drive);

ViStatus _VI_FUNC AVR3271_Copy_MemoryCard (ViSession instrID,
                                          ViBoolean parameter);

ViStatus _VI_FUNC AVR3271_Recall_File (ViSession instrID,
                                      ViBoolean drive,
                                      ViString fileName);
                                      
ViStatus _VI_FUNC AVR3271_Save_File (ViSession instrID,
                                    ViBoolean drive,
                                    ViString fileName);                                   
                                  


/*Standard Utility Functions*/

ViStatus _VI_FUNC AVR3271_WriteInstrData (ViSession instrID,
                                         ViString writeBuffer);

ViStatus _VI_FUNC AVR3271_readInstrData (ViSession instrID,
                                        ViInt32 numberBytesToRead,
                                        ViChar _VI_FAR readBuffer[],
                                        ViPInt32 numBytesRead);
                                      
ViStatus _VI_FUNC AVR3271_reset (ViSession instrSession);
                              
ViStatus _VI_FUNC AVR3271_selfTest (ViSession instrID,
                                   ViPInt16 selfTestResult,
                                   ViChar _VI_FAR selfTestMessage[]);
                                 
ViStatus _VI_FUNC AVR3271_errorQuery (ViSession instrID,
                                     ViPInt32 errorCode,
                                     ViChar _VI_FAR errorMessage[]);
                                   
ViStatus _VI_FUNC AVR3271_errorMessage (ViSession instrID,
                                       ViStatus statusCode,
                                       ViChar _VI_FAR message[]);
                                     
ViStatus _VI_FUNC AVR3271_revisionQuery (ViSession instrID,
                                        ViChar _VI_FAR instrumentDriverRevision[],
                                        ViChar _VI_FAR firmwareRevision[]);                           
                           
ViStatus _VI_FUNC AVR3271_close (ViSession instrID);


#if defined(__cplusplus) || defined(__cplusplus__)
}
#endif

/*===========================================================================*/
/* END INCLUDE FILE                                                          */
/*===========================================================================*/

#endif
